from __future__ import annotations

from django.apps import apps

from django_spire.exceptions import (
    DjangoSpireInvalidClassStringError,
    DjangoSpireMissingRequiredAppError
)


def app_is_installed(app_label: str) -> bool:
    return app_label in list(apps.app_configs.keys())


def check_required_apps(app_label: str) -> None:
    app_config = apps.get_app_config(app_label)

    for required_app_name in app_config.REQUIRED_APPS:
        if not app_is_installed(required_app_name):
            message = f'{app_label} requires {required_app_name} is be in the "INSTALLED_APPS" list before {app_label} in the django settings module.'
            raise DjangoSpireMissingRequiredAppError(message)


def get_class_from_string(class_string: str) -> type:
    class_parts = class_string.split('.')

    if len(class_parts) < 2:
        message = f'Class string {class_string} is not a valid class string.'
        raise DjangoSpireInvalidClassStringError(message)

    module_path = '.'.join(class_parts[:-1])
    class_name = class_parts[-1]

    module = __import__(module_path, fromlist=[class_name])

    return getattr(module, class_name)


def get_class_name_from_class(cls: type) -> str:
    return cls.__module__ + '.' + cls.__qualname__



from __future__ import annotations

from typing import TYPE_CHECKING

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.db.models import Q
from django.urls import reverse

from django_spire.contrib.service import BaseDjangoModelService
from django_spire.help_desk.exceptions import TicketEventNotificationTypeNotSupportedError
from django_spire.help_desk.enums import TicketEventType
from django_spire.notification.app.models import AppNotification
from django_spire.notification.choices import NotificationTypeChoices
from django_spire.notification.email.models import EmailNotification
from django_spire.notification.maps import NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP
from django_spire.notification.models import Notification

if TYPE_CHECKING:
    from django_spire.help_desk.models import HelpDeskTicket


class HelpDeskTicketNotificationService(BaseDjangoModelService['HelpDeskTicket']):
    obj: HelpDeskTicket

    def create_new_ticket_notifications(self):
        event_type = TicketEventType.NEW

        developers = User.objects.filter(
            email__in=[
                admin[1]
                for admin in settings.ADMINS
            ]
        )

        for notif_type in [NotificationTypeChoices.APP, NotificationTypeChoices.EMAIL]:
            self._create_ticket_event_notifications(
                users=developers,
                notification_type=notif_type,
                event_type=event_type,
            )

        managers = User.objects.filter(
            Q(groups__permissions__codename='delete_helpdeskticket')
        )

        self._create_ticket_event_notifications(
            users=managers,
            notification_type=NotificationTypeChoices.APP,
            event_type=event_type,
        )

    @staticmethod
    def _get_ticket_event_notification_title(event_type: TicketEventType) -> str:
        content_map = {
            TicketEventType.NEW: 'A new help desk ticket has been created',
        }

        return content_map[event_type]

    def _get_ticket_event_notification_body(
        self,
        event_type: TicketEventType,
        notification_type: NotificationTypeChoices,
    ) -> str:
        content_map = {
            TicketEventType.NEW: {
                NotificationTypeChoices.EMAIL: f"""
                    <h2>A new help desk ticket (#{self.obj.pk}) has been created.</h2>
                    <p>Priority: {self.obj.get_priority_display()}</p>
                    <p>Purpose: {self.obj.get_purpose_display()}</p>
                    <p>Description:</p>
                    <p>{self.obj.description}</p>
                """,
                NotificationTypeChoices.APP: (
                    f'Priority: {self.obj.get_priority_display()} - '
                    f'Purpose: {self.obj.get_purpose_display()}'
                ),
            }
        }

        try:
            return content_map[event_type][notification_type]
        except KeyError:
            raise TicketEventNotificationTypeNotSupportedError(event_type, notification_type) from None

    def _get_ticket_notification_url(
        self,
        notification_type: NotificationTypeChoices
    ) -> str:
        path = reverse('django_spire:help_desk:page:detail', kwargs={'pk': self.obj.pk})

        if notification_type == NotificationTypeChoices.EMAIL:
            site_host = Site.objects.get_current()
            return f'{site_host}{path}'

        return path

    def _create_ticket_event_notification_for_user(
        self,
        notification_type: NotificationTypeChoices,
        body: str,
        title: str,
        url: str,
        user: User,
    ) -> EmailNotification | AppNotification:
        base_notification = Notification(
            content_type=ContentType.objects.get_for_model(self.obj),
            object_id=self.obj.pk,
            url=url,
            title=title,
            body=body,
            type=notification_type,
            user=user,
        )

        if notification_type == NotificationTypeChoices.EMAIL:
            return EmailNotification(
                notification=base_notification,
                to_email_address=(
                    settings.DEVELOPMENT_EMAIL if settings.DEBUG else user.email
                )
            )

        if notification_type == NotificationTypeChoices.APP:
            return AppNotification(notification=base_notification)

        raise TicketEventNotificationTypeNotSupportedError(None, notification_type)

    def _create_ticket_event_notifications(
        self,
        users: list[User],
        notification_type: NotificationTypeChoices,
        event_type: TicketEventType
    ):
        title = self._get_ticket_event_notification_title(event_type)

        body = self._get_ticket_event_notification_body(
            event_type=event_type,
            notification_type=notification_type,
        )

        url = self._get_ticket_notification_url(notification_type)

        notifications = [
            self._create_ticket_event_notification_for_user(
                notification_type=notification_type,
                body=body,
                title=title,
                url=url,
                user=user,
            )
            for user in users
        ]

        Notification.objects.bulk_create(
            [
                notification.notification
                for notification in notifications
                if notification
            ]
        )

        NOTIFICATION_TYPE_CHOICE_TO_MODEL_MAP[notification_type].objects.bulk_create(
            notifications
        )



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.choices import (
    HelpDeskTicketPriorityChoices,
    HelpDeskTicketPurposeChoices,
    HelpDeskTicketStatusChoices,
)
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


class HelpDeskTicketModelTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.ticket = create_test_helpdesk_ticket()

    def test_str(self):
        assert str(self.ticket) == f'Ticket #{self.ticket.pk}'

    def test_default_status_is_ready(self):
        assert self.ticket.status == HelpDeskTicketStatusChoices.READY

    def test_get_priority_display(self):
        self.ticket.priority = HelpDeskTicketPriorityChoices.LOW
        assert self.ticket.get_priority_display() == 'Low'

        self.ticket.priority = HelpDeskTicketPriorityChoices.MEDIUM
        assert self.ticket.get_priority_display() == 'Medium'

        self.ticket.priority = HelpDeskTicketPriorityChoices.HIGH
        assert self.ticket.get_priority_display() == 'High'

        self.ticket.priority = HelpDeskTicketPriorityChoices.URGENT
        assert self.ticket.get_priority_display() == 'Urgent'

    def test_get_purpose_display(self):
        self.ticket.purpose = HelpDeskTicketPurposeChoices.APP
        assert self.ticket.get_purpose_display() == 'App'

        self.ticket.purpose = HelpDeskTicketPurposeChoices.COMPANY
        assert self.ticket.get_purpose_display() == 'Company'

    def test_get_status_display(self):
        self.ticket.status = HelpDeskTicketStatusChoices.READY
        assert self.ticket.get_status_display() == 'Ready'

        self.ticket.status = HelpDeskTicketStatusChoices.INPROGRESS
        assert self.ticket.get_status_display() == 'In Progress'

        self.ticket.status = HelpDeskTicketStatusChoices.DONE
        assert self.ticket.get_status_display() == 'Done'

    def test_created_by_relationship(self):
        assert self.ticket.created_by is not None
        assert self.ticket.created_by.pk is not None

    def test_created_datetime_auto_set(self):
        assert self.ticket.created_datetime is not None



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.contrib.ordering.services.service import OrderingService
from django_spire.contrib.service import BaseDjangoModelService
from django_spire.knowledge.entry.services.automation_service import EntryAutomationService
from django_spire.knowledge.entry.services.factory_service import EntryFactoryService
from django_spire.knowledge.entry.services.processor_service import EntryProcessorService
from django_spire.knowledge.entry.services.tag_service import EntryTagService
from django_spire.knowledge.entry.services.tool_service import EntryToolService
from django_spire.knowledge.entry.services.transformation_services import EntryTransformationService
from django_spire.knowledge.entry.version.models import EntryVersion

if TYPE_CHECKING:
    from django_spire.auth.user.models import AuthUser
    from django_spire.knowledge.entry.models import Entry


class EntryService(BaseDjangoModelService['Entry']):
    obj: Entry

    automation = EntryAutomationService()
    factory = EntryFactoryService()
    ordering = OrderingService()
    processor = EntryProcessorService()
    tag = EntryTagService()
    tool = EntryToolService()
    transformation = EntryTransformationService()

    def save_model_obj(self, author: AuthUser, **field_data) -> tuple[Entry, bool]:
        self.obj, created = super().save_model_obj(**field_data)

        if created:
            entry_version, _ = EntryVersion.services.save_model_obj(
                entry=self.obj,
                author=author
            )

            self.obj.current_version = entry_version
            self.obj.save()

        self.obj.ordering_services.processor.move_to_position(
            destination_objects=self.obj.collection.entries.active(),
            position=0 if created else self.obj.order,
        )

        return self.obj, created



from __future__ import annotations

from typing import TYPE_CHECKING

from django import template
from django.utils.html import escapejs
from django.utils.safestring import mark_safe

from django_spire.contrib.session.controller import SessionController

if TYPE_CHECKING:
    from django.template import Context
    from django.utils.safestring import SafeString


register = template.Library()


@register.simple_tag(takes_context=True)
def session_controller_to_json(context: Context, key: str) -> SafeString:
    request = context.get('request')
    controller = SessionController(request, key)
    data = controller.to_json()
    return mark_safe(escapejs(data))



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib.service import BaseDjangoModelService

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest

    from django_spire.knowledge.collection.models import Collection, CollectionGroup


class CollectionGroupFactoryService(BaseDjangoModelService['CollectionGroup']):
    obj: CollectionGroup

    def replace_groups(
        self,
        request: WSGIRequest,
        group_pks: list[int] | None,
        collection: Collection
    ) -> list[CollectionGroup]:
        if not AppAuthController('knowledge', request).can_change_collection_groups():
            return []

        if group_pks is None:
            return collection.groups.all().delete()

        old_collection_groups = list(collection.groups.all())

        new_collection_groups = [
            self.obj_class(auth_group_id=group_pk, collection=collection)
            for group_pk in group_pks
        ]

        self.obj_class.objects.bulk_create(new_collection_groups)

        for collection in old_collection_groups:
            collection.delete()

        return new_collection_groups



from __future__ import annotations

from django_spire.auth.user.models import AuthUser


def create_user(
    username: str,
    **kwargs
) -> AuthUser:
    user, _ = AuthUser.objects.get_or_create(
        username=username,
        defaults={
            **kwargs
        }
    )

    return user


def create_super_user(password: str = 'stratus', **kwargs) -> AuthUser:
    defaults = {
        'email': 'bobert@stratusadv.com',
        'first_name': 'Bob',
        'last_name': 'Robertson',
        'is_superuser': True,
        'is_staff': True,
        **kwargs
    }

    user, _ = AuthUser.objects.update_or_create(
        username='stratus',
        defaults=defaults
    )

    user.set_password(password)
    user.save()

    return user



from __future__ import annotations

from typing import TYPE_CHECKING

from dandy import Prompt

from django_spire.core.tag.intelligence.tag_set_bot import TagSetBot
from django_spire.core.tag.service.tag_service import BaseTagService

if TYPE_CHECKING:
    from django_spire.knowledge.entry.models import Entry


class EntryTagService(BaseTagService['Entry']):
    obj: Entry

    def process_and_set_tags(self):
        entry_prompt = Prompt()

        entry_prompt.sub_heading(self.obj.name)

        for version_block in self.obj.current_version.blocks.all():
            entry_prompt.text(f'{version_block.render_to_text()}')

        tag_set = TagSetBot().process(
            content=entry_prompt
        )

        self.set_tags_from_tag_set(
            tag_set=tag_set,
        )





from __future__ import annotations

from django.contrib.admin.sites import AdminSite

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.admin import NotificationAdmin
from django_spire.notification.models import Notification
from django_spire.notification.tests.factories import create_test_notification


class NotificationAdminTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.site = AdminSite()
        self.admin = NotificationAdmin(Notification, self.site)
        self.notification = create_test_notification()

    def test_list_display(self):
        expected = (
            'id', 'title', 'type', 'user', 'view_body_snippet', 'url_link', 'status',
            'status_message', 'priority', 'sent_datetime', 'publish_datetime',
            'content_type', 'object_id', 'is_deleted'
        )
        assert self.admin.list_display == expected

    def test_list_filter(self):
        expected = ('type',)
        assert self.admin.list_filter == expected

    def test_search_fields(self):
        expected = ('id', 'title', 'type')
        assert self.admin.search_fields == expected

    def test_view_body_snippet_truncates_long_body(self):
        self.notification.body = 'A' * 50
        result = self.admin.view_body_snippet(self.notification)
        assert len(result) <= 23
        assert result.endswith('...')

    def test_view_body_snippet_short_body(self):
        self.notification.body = 'Short'
        result = self.admin.view_body_snippet(self.notification)
        assert result == 'Short'

    def test_url_link_with_url(self):
        self.notification.url = 'https://example.com'
        result = self.admin.url_link(self.notification)
        assert 'href="https://example.com"' in result
        assert 'Link' in result

    def test_url_link_without_url(self):
        self.notification.url = ''
        result = self.admin.url_link(self.notification)
        assert result == 'No URL'



from __future__ import annotations

from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
from django_spire.knowledge.entry.version.block.data.heading_data import HeadingEditorBlockData
from django_spire.knowledge.entry.version.block.data.list.data import ListEditorBlockData
from django_spire.knowledge.entry.version.block.data.text_data import TextEditorBlockData


EDITOR_JS_BLOCK_DATA_MAP = {
    BlockTypeChoices.TEXT: TextEditorBlockData,
    BlockTypeChoices.HEADING: HeadingEditorBlockData,
    BlockTypeChoices.LIST: ListEditorBlockData,
}


EDITOR_JS_BLOCK_DATA_REVERSE_MAP = {
    value: key for key, value in EDITOR_JS_BLOCK_DATA_MAP.items()
}



from __future__ import annotations

import pytest

from django.db import models
from django.test import TestCase
from django.utils.timezone import now

from django_spire.contrib.seeding import DjangoModelSeeder


class Product(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField()
    sku = models.CharField(max_length=20, unique=True)
    price = models.DecimalField(max_digits=6, decimal_places=2)
    in_stock = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(null=True, blank=True)

    class Meta:
        app_label = 'tests'
        managed = False

    def __str__(self) -> str:
        return 'Product'


class ProductSeeder(DjangoModelSeeder):
    model_class = Product
    fields = {
        'name': ('llm', 'Clothing store'),
        'description': ('static', 'A great product for testing.'),
        'price': ('faker', 'pydecimal', {'left_digits': 2, 'right_digits': 2, 'positive': True}),
        'in_stock': True,
        'created_at': ('callable', now),
        'updated_at': 'exclude',
    }


class TestProductSeeder(TestCase):
    def test_assign_defaults(self) -> None:
        assert 'sku' in ProductSeeder.resolved_fields()

    def test_field_override_applies(self) -> None:
        override_time = now()

        result = ProductSeeder.seed(count=1, fields={'updated_at': ('static', override_time)})

        assert result[0].updated_at == override_time

    def test_raises_for_invalid_field(self) -> None:
        with pytest.raises(Exception) as exc_info:
            ProductSeeder.seed(count=1, fields={'not_a_field': ('static', 'value')})

        assert 'not_a_field' in str(exc_info.value)

    def test_seeds_three_objects(self) -> None:
        result = ProductSeeder.seed(count=3)

        assert len(result) == 3

        for obj in result:
            assert isinstance(obj, Product)



from __future__ import annotations

import json

from django.forms import Widget
from django.template import loader
from django.utils.safestring import mark_safe


class TaggingWidget(Widget):
    template_name = 'django_spire/comment/widget/tagging_widget.html'
    user_list = None
    information = None

    def __init__(self, user_list, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_list = user_list

    def get_context(self, name: str, value, attrs):
        context_data = super().get_context(name, value, attrs)
        context_data['user_list'] = json.dumps(self.user_list)
        # context_data['information'] = self.information
        return context_data

    def render(self, name: str, value, attrs = None, renderer = None):
        context = self.get_context(name, value, attrs)
        template = loader.get_template(self.template_name).render(context)
        return mark_safe(template)



from __future__ import annotations

import pytest

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.sms.tools import format_to_international_phone_number


class TestSMSTools(BaseTestCase):
    def test_local_to_international_phone_number(self):
        phone_numbers = (
            '622 415 2736',
            '881-553-4599 x702',
            '+1 (588) 841-1582',
            '+1 (205) 461-5871',
            '742-180-9582',
            '691 287 8913',
            '(461) 107-2210 x009',
            '836-958-3866 x265',
            '615.332.9689',
            '919-286-3636 x939',
            '368.447-9514',
            '4563219876'
        )

        expected_phone_numbers = (
            '+16224152736',
            '+18815534599',
            '+15888411582',
            '+12054615871',
            '+17421809582',
            '+16912878913',
            '+14611072210',
            '+18369583866',
            '+16153329689',
            '+19192863636',
            '+13684479514',
            '+14563219876'
        )

        formatted_phone_numbers = tuple(
            format_to_international_phone_number(phone_number)
            for phone_number in phone_numbers
        )
        assert formatted_phone_numbers == expected_phone_numbers

    def test_invalid_phone_number(self):
        phone_numbers = (
            'ABC-DEF-GHIJ',
            '123',
            '1',
            '12345678901234567890',
            '',
            '32145698765',
            '403000123',
            '36800012',
        )

        for phone_number in phone_numbers:
            with pytest.raises(ValueError):
                format_to_international_phone_number(phone_number)

    def test_format_with_country_code(self):
        result = format_to_international_phone_number('5551234567', country_code='1')
        assert result == '+15551234567'

    def test_format_already_has_country_code(self):
        result = format_to_international_phone_number('15551234567', country_code='1')
        assert result == '+15551234567'



from __future__ import annotations

from django.contrib.contenttypes.models import ContentType
from django.utils.timezone import now

from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.history.viewed.models import Viewed
from django_spire.notification.app.models import AppNotification
from django_spire.notification.app.tests.factories import create_test_app_notification
from django_spire.notification.choices import NotificationStatusChoices


class AppNotificationQuerySetTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.user = create_user(username='test_app_notification_user')
        self.app_notification = create_test_app_notification(user=self.user)

    def _create_viewed_for_notification(self, app_notification: AppNotification, user):
        content_type = ContentType.objects.get_for_model(AppNotification)
        Viewed.objects.create(
            user=user,
            object_id=app_notification.id,
            content_type=content_type
        )

    def test_by_user(self):
        other_user = create_user(username='other_user')
        other_notification = create_test_app_notification(user=other_user)

        result = AppNotification.objects.by_user(self.user)
        assert self.app_notification in result
        assert other_notification not in result

    def test_by_users(self):
        other_user = create_user(username='other_user')
        other_notification = create_test_app_notification(user=other_user)

        result = AppNotification.objects.by_users([self.user, other_user])
        assert self.app_notification in result
        assert other_notification in result

    def test_is_sent(self):
        pending_notification = create_test_app_notification(
            user=self.user,
            status=NotificationStatusChoices.PENDING
        )

        result = AppNotification.objects.is_sent()
        assert self.app_notification in result
        assert pending_notification not in result

    def test_exclude_viewed_by_user(self):
        self._create_viewed_for_notification(self.app_notification, self.user)

        result = AppNotification.objects.exclude_viewed_by_user(self.user)
        assert self.app_notification not in result

    def test_annotate_is_viewed_by_user_false(self):
        result = AppNotification.objects.annotate_is_viewed_by_user(self.user).first()
        assert result.viewed is False

    def test_annotate_is_viewed_by_user_true(self):
        self._create_viewed_for_notification(self.app_notification, self.user)

        result = AppNotification.objects.annotate_is_viewed_by_user(self.user).first()
        assert result.viewed is True

    def test_ordered_by_priority_and_sent_datetime(self):
        same_time = now()
        high_priority = create_test_app_notification(
            user=self.user,
            priority='high',
            sent_datetime=same_time
        )
        low_priority = create_test_app_notification(
            user=self.user,
            priority='low',
            sent_datetime=same_time
        )

        result = list(
            AppNotification.objects
            .filter(pk__in=[high_priority.pk, low_priority.pk])
            .ordered_by_priority_and_sent_datetime()
        )

        assert result[0].pk == high_priority.pk
        assert result[1].pk == low_priority.pk



from __future__ import annotations

from django_spire.auth.group.models import AuthGroup
from django_spire.auth.permissions.permissions import GroupPermissions, ModelPermission
from django_spire.auth.permissions.tools import (
    generate_group_perm_data,
    generate_model_key_permission_map,
    generate_model_permissions,
    generate_special_role_data,
    generate_user_perm_data,
)
from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase


class GenerateModelPermissionsTestCase(BaseTestCase):
    def test_returns_list(self) -> None:
        result = generate_model_permissions()
        assert isinstance(result, list)

    def test_contains_model_permission_instances(self) -> None:
        result = generate_model_permissions()
        for item in result:
            assert isinstance(item, ModelPermission)

    def test_contains_group_permission(self) -> None:
        result = generate_model_permissions()
        names = [mp.name for mp in result]
        assert 'group' in names

    def test_contains_user_permission(self) -> None:
        result = generate_model_permissions()
        names = [mp.name for mp in result]
        assert 'user' in names

    def test_all_have_required_attributes(self) -> None:
        result = generate_model_permissions()
        for mp in result:
            assert hasattr(mp, 'name')
            assert hasattr(mp, 'model_class_path')
            assert hasattr(mp, 'is_proxy_model')

    def test_model_class_path_is_string(self) -> None:
        result = generate_model_permissions()
        for mp in result:
            assert isinstance(mp.model_class_path, str)

    def test_name_is_string(self) -> None:
        result = generate_model_permissions()
        for mp in result:
            assert isinstance(mp.name, str)

    def test_is_proxy_model_is_bool(self) -> None:
        result = generate_model_permissions()
        for mp in result:
            assert isinstance(mp.is_proxy_model, bool)

    def test_result_not_empty(self) -> None:
        result = generate_model_permissions()
        assert len(result) > 0

    def test_result_is_consistent(self) -> None:
        result1 = generate_model_permissions()
        result2 = generate_model_permissions()
        names1 = sorted([mp.name for mp in result1])
        names2 = sorted([mp.name for mp in result2])
        assert names1 == names2


class GenerateModelKeyPermissionMapTestCase(BaseTestCase):
    def test_returns_dict(self) -> None:
        result = generate_model_key_permission_map()
        assert isinstance(result, dict)

    def test_keys_are_lowercase(self) -> None:
        result = generate_model_key_permission_map()
        for key in result:
            assert key == key.lower()

    def test_contains_group_key(self) -> None:
        result = generate_model_key_permission_map()
        assert 'group' in result

    def test_contains_user_key(self) -> None:
        result = generate_model_key_permission_map()
        assert 'user' in result

    def test_values_are_model_permissions(self) -> None:
        result = generate_model_key_permission_map()
        for value in result.values():
            assert isinstance(value, ModelPermission)

    def test_keys_match_permission_names(self) -> None:
        result = generate_model_key_permission_map()
        for key, value in result.items():
            assert key == value.name.lower()

    def test_result_not_empty(self) -> None:
        result = generate_model_key_permission_map()
        assert len(result) > 0

    def test_keys_are_strings(self) -> None:
        result = generate_model_key_permission_map()
        for key in result.keys():
            assert isinstance(key, str)


class GenerateUserPermDataTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(username='testuser')

    def test_returns_list(self) -> None:
        result = generate_user_perm_data(self.user)
        assert isinstance(result, list)

    def test_contains_app_name_and_level(self) -> None:
        result = generate_user_perm_data(self.user)
        for item in result:
            assert 'app_name' in item
            assert 'level_verbose' in item

    def test_default_level_is_none(self) -> None:
        result = generate_user_perm_data(self.user)
        for item in result:
            assert item['level_verbose'] == 'None'

    def test_user_with_permissions(self) -> None:
        group = AuthGroup.objects.create(name='Test Group')
        self.user.groups.add(group)
        model_permission = ModelPermission(
            name='group',
            model_class_path='django_spire.auth.group.models.AuthGroup',
            is_proxy_model=True
        )
        group_perms = GroupPermissions(group, model_permission)
        group_perms.update_perms(3)

        result = generate_user_perm_data(self.user)
        group_perm = next((p for p in result if p['app_name'] == 'Group'), None)
        assert group_perm is not None
        assert group_perm['level_verbose'] == 'Change'

    def test_returns_data_for_all_registered_permissions(self) -> None:
        result = generate_user_perm_data(self.user)
        model_permissions = generate_model_permissions()
        assert len(result) == len(model_permissions)

    def test_app_name_is_capitalized(self) -> None:
        result = generate_user_perm_data(self.user)
        for item in result:
            assert item['app_name'][0].isupper()

    def test_level_verbose_is_string(self) -> None:
        result = generate_user_perm_data(self.user)
        for item in result:
            assert isinstance(item['level_verbose'], str)

    def test_result_items_are_dicts(self) -> None:
        result = generate_user_perm_data(self.user)
        for item in result:
            assert isinstance(item, dict)

    def test_user_with_delete_permission(self) -> None:
        group = AuthGroup.objects.create(name='Delete Group')
        self.user.groups.add(group)
        model_permission = ModelPermission(
            name='group',
            model_class_path='django_spire.auth.group.models.AuthGroup',
            is_proxy_model=True
        )
        group_perms = GroupPermissions(group, model_permission)
        group_perms.update_perms(4)

        result = generate_user_perm_data(self.user)
        group_perm = next((p for p in result if p['app_name'] == 'Group'), None)
        assert group_perm is not None
        assert group_perm['level_verbose'] == 'Delete'


class GenerateGroupPermDataTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.group = AuthGroup.objects.create(name='Test Group')

    def test_returns_list(self) -> None:
        result = generate_group_perm_data(self.group)
        assert isinstance(result, list)

    def test_contains_required_keys(self) -> None:
        result = generate_group_perm_data(self.group)
        for item in result:
            assert 'app_name' in item
            assert 'verbose_app_name' in item
            assert 'level_verbose' in item

    def test_without_special_role(self) -> None:
        result = generate_group_perm_data(self.group, with_special_role=False)
        for item in result:
            assert 'special_role_data' not in item

    def test_with_special_role(self) -> None:
        result = generate_group_perm_data(self.group, with_special_role=True)
        for item in result:
            assert 'special_role_data' in item

    def test_default_level_is_none(self) -> None:
        result = generate_group_perm_data(self.group)
        for item in result:
            assert item['level_verbose'] == 'None'

    def test_group_with_permissions(self) -> None:
        model_permission = ModelPermission(
            name='group',
            model_class_path='django_spire.auth.group.models.AuthGroup',
            is_proxy_model=True
        )
        group_perms = GroupPermissions(self.group, model_permission)
        group_perms.update_perms(4)

        result = generate_group_perm_data(self.group)
        group_perm = next((p for p in result if p['app_name'] == 'Group'), None)
        assert group_perm is not None
        assert group_perm['level_verbose'] == 'Delete'

    def test_app_name_is_capitalized(self) -> None:
        result = generate_group_perm_data(self.group)
        for item in result:
            assert item['app_name'][0].isupper()

    def test_result_items_are_dicts(self) -> None:
        result = generate_group_perm_data(self.group)
        for item in result:
            assert isinstance(item, dict)

    def test_level_verbose_is_string(self) -> None:
        result = generate_group_perm_data(self.group)
        for item in result:
            assert isinstance(item['level_verbose'], str)

    def test_returns_data_for_all_registered_permissions(self) -> None:
        result = generate_group_perm_data(self.group)
        model_permissions = generate_model_permissions()
        assert len(result) == len(model_permissions)

    def test_special_role_data_is_list(self) -> None:
        result = generate_group_perm_data(self.group, with_special_role=True)
        for item in result:
            assert isinstance(item['special_role_data'], list)


class GenerateSpecialRoleDataTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.group = AuthGroup.objects.create(name='Test Group')
        self.model_permission = ModelPermission(
            name='group',
            model_class_path='django_spire.auth.group.models.AuthGroup',
            is_proxy_model=True
        )
        self.group_permissions = GroupPermissions(self.group, self.model_permission)

    def test_returns_list(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        assert isinstance(result, list)

    def test_special_role_dict_structure(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert 'name' in item
            assert 'codename' in item
            assert 'has_access' in item

    def test_has_access_default_false(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert item['has_access'] is False

    def test_codename_starts_with_can(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert item['codename'].startswith('can')

    def test_name_is_string(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert isinstance(item['name'], str)

    def test_codename_is_string(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert isinstance(item['codename'], str)

    def test_has_access_is_bool(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert isinstance(item['has_access'], bool)

    def test_result_items_are_dicts(self) -> None:
        result = generate_special_role_data(self.group_permissions)
        for item in result:
            assert isinstance(item, dict)



from __future__ import annotations

from django.urls import path

from django_spire.notification.app.views import page_views


app_name = 'django_spire_notification'

urlpatterns = [
    path('django_spire/notification/list/',
        view=page_views.app_notification_list_view,
        name='list')
]



from __future__ import annotations

from django.contrib.admin.sites import AdminSite
from django.contrib.contenttypes.models import ContentType
from django.test import override_settings

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.file.admin import FileAdmin
from django_spire.file.models import File
from django_spire.file.tests.factories import create_test_file
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


STORAGES_OVERRIDE = {
    'default': {
        'BACKEND': 'django.core.files.storage.FileSystemStorage',
    },
    'staticfiles': {
        'BACKEND': 'django.contrib.staticfiles.storage.StaticFilesStorage',
    },
}


@override_settings(STORAGES=STORAGES_OVERRIDE)
class FileAdminTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.site = AdminSite()
        self.admin = FileAdmin(File, self.site)
        self.file = create_test_file()

    def test_list_display(self):
        expected = ('id', 'name', 'type', 'size', 'content_object_link', 'file_link')

        assert self.admin.list_display == expected

    def test_list_filter(self):
        assert self.admin.list_filter == ('type',)

    def test_search_fields(self):
        assert self.admin.search_fields == ('id', 'name', 'type')

    def test_ordering(self):
        assert self.admin.ordering == ('-id',)

    def test_content_object_link_no_related_object(self):
        result = self.admin.content_object_link(self.file)

        assert result == 'No Related Object'

    def test_content_object_link_with_related_object(self):
        ticket = create_test_helpdesk_ticket()
        content_type = ContentType.objects.get_for_model(ticket.__class__)
        self.file.content_type = content_type
        self.file.object_id = ticket.pk
        self.file.save()

        result = self.admin.content_object_link(self.file)

        assert 'href=' in result
        assert str(ticket) in result

    def test_file_link(self):
        result = self.admin.file_link(self.file)

        assert f'href="{self.file.file.url}"' in result
        assert self.file.name in result
        assert 'download' in result



from django.urls import path

from django_spire.ai.chat.views.template import template_views


app_name = 'template'

urlpatterns = [
    path("<int:pk>/confirm_delete", template_views.confirm_delete_view, name="confirm_delete"),
    path("load/", template_views.dialog_widget_view, name="load"),
    path("recent/", template_views.recent_chats_widget_view, name="recent"),
    path("search/", template_views.search_chats_results_widget_view, name="search"),
]



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.collection.tests.factories import (
    create_test_collection,
    create_test_collection_group,
)


class CollectionModelTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection = create_test_collection()

    def test_str(self):
        assert str(self.collection) == self.collection.name

    def test_name_short_truncates(self):
        long_name = 'A' * 100
        collection = create_test_collection(name=long_name)
        assert len(collection.name_short) <= 35

    def test_name_short_no_truncate(self):
        short_name = 'Short'
        collection = create_test_collection(name=short_name)
        assert collection.name_short == short_name

    def test_top_level_parent_returns_self_when_no_parent(self):
        assert self.collection.top_level_parent == self.collection

    def test_top_level_parent_returns_root(self):
        child = create_test_collection(parent=self.collection, name='Child')
        grandchild = create_test_collection(parent=child, name='Grandchild')
        assert grandchild.top_level_parent == self.collection

    def test_base_breadcrumb(self):
        breadcrumbs = self.collection.base_breadcrumb()
        assert breadcrumbs is not None


class CollectionGroupModelTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection_group = create_test_collection_group()

    def test_str(self):
        expected = f'{self.collection_group.collection.name} - {self.collection_group.auth_group.name}'
        assert str(self.collection_group) == expected



from __future__ import annotations

from django.urls import include, path


app_name = 'group'

urlpatterns = [
    path('page/', include('django_spire.auth.group.urls.page_urls', namespace='page')),
    path('form/', include('django_spire.auth.group.urls.form_urls', namespace='form')),
    path('json/', include('django_spire.auth.group.urls.json_urls', namespace='json')),
]



from typing import Callable

from django.core.handlers.wsgi import WSGIRequest
from django.http.response import HttpResponse


def bearer_token_auth_required(token: str) -> Callable[[Callable], Callable]:
    def decorator_wrapper(view_func: Callable) -> Callable:
        def view_wrapper(request: WSGIRequest, *args, **kwargs) -> HttpResponse:
            if request.user.is_authenticated:
                return view_func(request, *args, **kwargs)

            auth_header_parts = request.META.get('HTTP_AUTHORIZATION', '').split(' ')

            if auth_header_parts[0].lower() == 'bearer':
                if auth_header_parts[1] == token:
                    return view_func(request, *args, **kwargs)

            return HttpResponse(status=401)

        return view_wrapper

    return decorator_wrapper


from __future__ import annotations

from django.contrib.auth import get_user
from django.urls import reverse

from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase


class LoginViewTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.client.logout()
        self.user = create_user(username='testuser', email='test@example.com')
        self.user.set_password('testpassword123')
        self.user.save()

    def test_login_page_loads(self) -> None:
        response = self.client.get(reverse('django_spire:auth:admin:login'))
        assert response.status_code == 200

    def test_login_with_valid_credentials(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'testpassword123'}
        )
        assert response.status_code == 302

    def test_login_with_invalid_credentials(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'wrongpassword'}
        )
        assert response.status_code == 200

    def test_login_with_nonexistent_user(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'nonexistent', 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_empty_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': '', 'password': 'testpassword123'}
        )
        assert response.status_code == 200

    def test_login_with_empty_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': ''}
        )
        assert response.status_code == 200

    def test_login_inactive_user(self) -> None:
        self.user.is_active = False
        self.user.save()
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'testpassword123'}
        )
        assert response.status_code == 200

    def test_login_with_whitespace_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': '   ', 'password': 'testpassword123'}
        )
        assert response.status_code == 200

    def test_login_case_sensitive_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'TESTUSER', 'password': 'testpassword123'}
        )
        assert response.status_code == 200

    def test_login_case_sensitive_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'TESTPASSWORD123'}
        )
        assert response.status_code == 200

    def test_login_sets_session(self) -> None:
        self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'testpassword123'}
        )
        user = get_user(self.client)
        assert user.is_authenticated

    def test_login_with_sql_injection_attempt(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': "'; DROP TABLE users; --", 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_xss_attempt(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': '<script>alert("xss")</script>', 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_very_long_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'a' * 1000, 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_very_long_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'a' * 1000}
        )
        assert response.status_code == 200

    def test_login_with_unicode_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'tëstüsér', 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_unicode_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'pässwörd'}
        )
        assert response.status_code == 200

    def test_login_get_request_does_not_authenticate(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'testuser', 'password': 'testpassword123'}
        )
        user = get_user(self.client)
        assert not user.is_authenticated
        assert response.status_code == 200

    def test_login_with_null_bytes(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'test\x00user', 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_newline_in_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'test\nuser', 'password': 'password'}
        )
        assert response.status_code == 200

    def test_login_with_tab_in_username(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:login'),
            data={'username': 'test\tuser', 'password': 'password'}
        )
        assert response.status_code == 200


class PasswordChangeViewTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(username='testuser')
        self.user.set_password('oldpassword123')
        self.user.save()
        self.client.force_login(self.user)

    def test_password_change_page_loads(self) -> None:
        response = self.client.get(reverse('django_spire:auth:admin:password_change'))
        assert response.status_code == 200

    def test_password_change_success(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': 'newpassword456',
                'new_password2': 'newpassword456'
            }
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert self.user.check_password('newpassword456')

    def test_password_change_wrong_old_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'wrongoldpassword',
                'new_password1': 'newpassword456',
                'new_password2': 'newpassword456'
            }
        )
        assert response.status_code == 200
        self.user.refresh_from_db()
        assert self.user.check_password('oldpassword123')

    def test_password_change_mismatched_passwords(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': 'newpassword456',
                'new_password2': 'differentpassword789'
            }
        )
        assert response.status_code == 200

    def test_password_change_empty_new_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': '',
                'new_password2': ''
            }
        )
        assert response.status_code == 200

    def test_password_change_requires_login(self) -> None:
        self.client.logout()
        response = self.client.get(reverse('django_spire:auth:admin:password_change'))
        assert response.status_code == 302

    def test_password_change_same_as_old(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': 'oldpassword123',
                'new_password2': 'oldpassword123'
            }
        )
        assert response.status_code in (200, 302)

    def test_password_change_with_unicode(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': 'nëwpässwörd456',
                'new_password2': 'nëwpässwörd456'
            }
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert self.user.check_password('nëwpässwörd456')

    def test_password_change_very_long_password(self) -> None:
        long_password = 'a' * 128
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': long_password,
                'new_password2': long_password
            }
        )
        assert response.status_code == 302

    def test_password_change_with_spaces(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': 'new password 456',
                'new_password2': 'new password 456'
            }
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert self.user.check_password('new password 456')

    def test_password_change_preserves_session(self) -> None:
        self.client.post(
            reverse('django_spire:auth:admin:password_change'),
            data={
                'old_password': 'oldpassword123',
                'new_password1': 'newpassword456',
                'new_password2': 'newpassword456'
            }
        )
        user = get_user(self.client)
        assert user.is_authenticated


class PasswordChangeDoneViewTestCase(BaseTestCase):
    def test_password_change_done_page_loads(self) -> None:
        response = self.client.get(reverse('django_spire:auth:admin:password_change_done'))
        assert response.status_code == 200


class PasswordResetViewTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(username='testuser', email='test@example.com')

    def test_password_reset_page_loads(self) -> None:
        response = self.client.get(reverse('django_spire:auth:admin:password_reset'))
        assert response.status_code == 200

    def test_password_reset_with_valid_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': 'test@example.com'}
        )
        assert response.status_code == 302

    def test_password_reset_with_nonexistent_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': 'nonexistent@example.com'}
        )
        assert response.status_code == 302

    def test_password_reset_with_empty_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': ''}
        )
        assert response.status_code == 200

    def test_password_reset_with_invalid_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': 'invalid-email'}
        )
        assert response.status_code == 200

    def test_password_reset_with_uppercase_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': 'TEST@EXAMPLE.COM'}
        )
        assert response.status_code == 302

    def test_password_reset_with_mixed_case_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': 'Test@Example.Com'}
        )
        assert response.status_code == 302

    def test_password_reset_with_whitespace_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': '  test@example.com  '}
        )
        assert response.status_code in (200, 302)

    def test_password_reset_inactive_user(self) -> None:
        self.user.is_active = False
        self.user.save()
        response = self.client.post(
            reverse('django_spire:auth:admin:password_reset'),
            data={'email': 'test@example.com'}
        )
        assert response.status_code == 302


class PasswordResetDoneViewTestCase(BaseTestCase):
    def test_password_reset_done_page_loads(self) -> None:
        response = self.client.get(reverse('django_spire:auth:admin:password_reset_done'))
        assert response.status_code == 200


class LogoutRedirectViewTestCase(BaseTestCase):
    def test_logout_redirects(self) -> None:
        response = self.client.get(reverse('django_spire:auth:redirect:logout'))
        assert response.status_code == 302

    def test_logout_logs_out_user(self) -> None:
        self.client.get(reverse('django_spire:auth:redirect:logout'))
        response = self.client.get(reverse('django_spire:auth:admin:password_change'))
        assert response.status_code != 200

    def test_logout_redirects_to_login(self) -> None:
        response = self.client.get(reverse('django_spire:auth:redirect:logout'))
        assert 'login' in response.url.lower()

    def test_logout_clears_session(self) -> None:
        self.client.get(reverse('django_spire:auth:redirect:logout'))
        user = get_user(self.client)
        assert not user.is_authenticated


class LoginRedirectViewTestCase(BaseTestCase):
    def test_login_redirect_for_authenticated_user(self) -> None:
        response = self.client.get(reverse('django_spire:auth:redirect:login'))
        assert response.status_code == 302

    def test_login_redirect_for_unauthenticated_user(self) -> None:
        self.client.logout()
        response = self.client.get(reverse('django_spire:auth:redirect:login'))
        assert response.status_code == 302



from __future__ import annotations


class DjangoSpireError(Exception):
    pass


class DjangoSpireConfigurationError(DjangoSpireError):
    pass


class DjangoSpireInvalidClassStringError(DjangoSpireError):
    pass


class DjangoSpireMissingRequiredAppError(DjangoSpireError):
    pass



from __future__ import annotations

from django.urls import path

from django_spire.auth.user.views import page_views


app_name = 'page'

urlpatterns = [
    path('user/<int:pk>/detail/', page_views.detail_view, name='detail'),
    path('user/list/', page_views.list_view, name='list'),
]



from __future__ import annotations

from django.db import models
from django.contrib.auth.models import User

from django_spire.help_desk.querysets import HelpDeskTicketQuerySet
from django_spire.help_desk.services.service import HelpDeskTicketService
from django_spire.history.mixins import HistoryModelMixin

from django_spire.help_desk import choices


class HelpDeskTicket(HistoryModelMixin):
    created_by = models.ForeignKey(
        User,
        on_delete=models.PROTECT,
        related_name='created_help_desk_tickets',
        related_query_name='created_help_desk_ticket',
        editable=False,
        verbose_name='Created By'
    )
    purpose = models.CharField(
        max_length=4,
        choices=choices.HelpDeskTicketPurposeChoices.choices,
        verbose_name='Purpose'
    )
    priority = models.CharField(
        max_length=4,
        choices=choices.HelpDeskTicketPriorityChoices.choices,
        verbose_name='Priority'
    )
    status = models.CharField(
        max_length=4,
        choices=choices.HelpDeskTicketStatusChoices.choices,
        default=choices.HelpDeskTicketStatusChoices.READY,
        verbose_name='Status'
    )
    description = models.TextField()

    objects = HelpDeskTicketQuerySet.as_manager()
    services = HelpDeskTicketService()

    def __str__(self):
        return f'Ticket #{self.pk}'

    class Meta:
        db_table = 'django_spire_help_desk_ticket'
        verbose_name = 'Help Desk Ticket'
        verbose_name_plural = 'Help Desk Tickets'



from __future__ import annotations

from pathlib import Path

from dandy import Prompt


_RELATIVE_BASE_DIR = Path(Path(__file__).parent.parent.parent.parent.resolve())
dandy_prompting_path = Path(_RELATIVE_BASE_DIR, '.venv/Lib/site-packages/dandy/llm/prompt/prompt.py')
dandy_tutorial_path = Path(_RELATIVE_BASE_DIR, 'django_spire/ai/prompt/docs/prompt_tutorial.md')


def dandy_prompt_python_file_instruction_bot_prompt():
    return (
        Prompt()
        .heading('Dandy Prompt Python Converter')
        .text('You are an expert Python developer specializing in converting natural language prompts into structured Dandy Prompt class implementations. Your task is to analyze user-provided prompts and convert them into well-structured Python code using the Dandy Prompt class methods, while preserving the original content and intent of the prompt.')
        .line_break()

        .heading('Goal')
        .text('Convert the user\'s natural language prompt into Python code using the Dandy Prompt class.')
        .line_break()

        .heading('Rules and Best Practices')
        .text('RETURN PURE PYTHON. NO MARKDOWN. SINGLE PROMPT CLASS.')
        .text('1. **Preserve All Content**: Include every word and character from the original prompt')
        .text('2. **Use Appropriate Methods**: Choose the most semantically appropriate Dandy method for each element')
        .text('3. **Maintain Order**: Keep the elements in the same order as the original prompt')
        .text('4. **Follow Python Style**: Use proper Python formatting, including indentation and line breaks')
        .text('5. **Prefer Method Chaining**: Use the chained method style (method1().method2()) for cleaner code')
        .text('6. **Handle Special Characters**: Properly escape quotes and special characters in strings')
        .text('7. **Preserve Whitespace**: Pay attention to blank lines and indentation in the original prompt')
        .text('8. **Use Parentheses for Clarity**: Wrap multi-line method chains in parentheses')

        .heading('Output Format')
        .text('Return a Python file containing the Dandy Prompt class implementation.')
        .heading('Prompt Tutorial')
        .file(dandy_tutorial_path)
        .heading('Dandy Prompt Class Source Code')
        .file(dandy_prompting_path)
    )


def text_to_markdown_instruction_bot_prompt():
    return (
        Prompt()
        .heading('Markdown Conversion Expert')
        .text('You are an expert in converting plain text into well-formatted Markdown. Your task is to analyze user-provided text and convert it into proper Markdown format, while preserving all original content and enhancing its presentation.')
        .line_break()

        .heading('Goal')
        .text('Convert the user\'s plain text into properly formatted Markdown with appropriate headings, bold text, italics, and other formatting elements.')
        .line_break()

        .heading('Rules and Best Practices')
        .ordered_list([
            'Preserve All Content: Include every word and character from the original text',
            'Infer Structure: Identify headings, lists, emphasis, and other structural elements from context',
            'Apply Proper Markdown: Use standard Markdown syntax (# for headings, ** for bold, * for italics, etc.)',
            'Maintain Hierarchy: Ensure proper heading levels (h1, h2, h3, etc.) based on content hierarchy',
            'Format Lists: Convert bullet points and numbered lists to proper Markdown list format',
            'Enhance Readability: Add appropriate line breaks and spacing for better readability',
            'Preserve Emphasis: Convert emphasized text (ALL CAPS, underlined, etc.) to appropriate Markdown formatting',
            'Add Code Formatting: Identify and properly format code snippets or technical terms',
            'Create Links: Convert URLs to proper Markdown link format',
            'Table Formatting: Convert tabular data into Markdown tables when appropriate'
        ])
        .line_break()

        .heading('Output Format')
        .text('Return the converted Markdown content and a suitable filename with .md extension.')
    )


def dandy_prompt_python_file_input_prompt(user_prompt: str):
    return (
        Prompt()
        .heading('User Prompt:')
        .text(user_prompt)
    )


def text_to_markdown_input_prompt(user_text: str):
    return (
        Prompt()
        .heading('User Text for Markdown Conversion:')
        .text(user_text)
    )



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.choices import (
    NotificationPriorityChoices,
    NotificationStatusChoices,
    NotificationTypeChoices,
)


class NotificationTypeChoicesTests(BaseTestCase):
    def test_app_value(self):
        assert NotificationTypeChoices.APP == 'app'

    def test_email_value(self):
        assert NotificationTypeChoices.EMAIL == 'email'

    def test_push_value(self):
        assert NotificationTypeChoices.PUSH == 'push'

    def test_sms_value(self):
        assert NotificationTypeChoices.SMS == 'sms'

    def test_choices_count(self):
        assert len(NotificationTypeChoices.choices) == 4


class NotificationStatusChoicesTests(BaseTestCase):
    def test_pending_value(self):
        assert NotificationStatusChoices.PENDING == 'pending'

    def test_processing_value(self):
        assert NotificationStatusChoices.PROCESSING == 'processing'

    def test_sent_value(self):
        assert NotificationStatusChoices.SENT == 'sent'

    def test_errored_value(self):
        assert NotificationStatusChoices.ERRORED == 'errored'

    def test_failed_value(self):
        assert NotificationStatusChoices.FAILED == 'failed'

    def test_choices_count(self):
        assert len(NotificationStatusChoices.choices) == 5


class NotificationPriorityChoicesTests(BaseTestCase):
    def test_low_value(self):
        assert NotificationPriorityChoices.LOW == 'low'

    def test_medium_value(self):
        assert NotificationPriorityChoices.MEDIUM == 'medium'

    def test_high_value(self):
        assert NotificationPriorityChoices.HIGH == 'high'

    def test_choices_count(self):
        assert len(NotificationPriorityChoices.choices) == 3



from __future__ import annotations

from django.apps import apps

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.notification.email.apps import NotificationEmailConfig


class NotificationEmailConfigTests(BaseTestCase):
    def test_app_name(self):
        assert NotificationEmailConfig.name == 'django_spire.notification.email'

    def test_app_label(self):
        assert NotificationEmailConfig.label == 'django_spire_notification_email'

    def test_default_auto_field(self):
        assert NotificationEmailConfig.default_auto_field == 'django.db.models.BigAutoField'

    def test_required_apps(self):
        expected = ('django_spire_core', 'django_spire_notification')
        assert NotificationEmailConfig.REQUIRED_APPS == expected

    def test_app_is_installed(self):
        assert apps.is_installed('django_spire.notification.email')



from __future__ import annotations

from django_spire.auth.user.factories import register_new_user
from django_spire.auth.user.models import AuthUser
from django_spire.core.tests.test_cases import BaseTestCase


class RegisterNewUserTestCase(BaseTestCase):
    def test_creates_user(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert isinstance(user, AuthUser) is True
        assert AuthUser.objects.filter(pk=user.pk).exists() is True

    def test_sets_correct_attributes(self) -> None:
        user = register_new_user(
            first_name='John',
            last_name='Doe',
            email='john.doe@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.first_name == 'John'
        assert user.last_name == 'Doe'
        assert user.email == 'john.doe@example.com'

    def test_username_is_lowercase_email(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='Test.User@Example.COM',
            password='securepassword123'  # noqa: S106
        )
        assert user.username == 'test.user@example.com'
        assert user.email == 'test.user@example.com'

    def test_password_is_set(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.check_password('securepassword123') is True

    def test_password_is_hashed(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.password != 'securepassword123'  # noqa: S106

    def test_user_is_active_by_default(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.is_active is True

    def test_user_is_not_staff_by_default(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.is_staff is False

    def test_user_is_not_superuser_by_default(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.is_superuser is False

    def test_empty_first_name(self) -> None:
        user = register_new_user(
            first_name='',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.first_name == ''

    def test_empty_last_name(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.last_name == ''

    def test_unicode_names(self) -> None:
        user = register_new_user(
            first_name='Tëst',
            last_name='Üser',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.first_name == 'Tëst'
        assert user.last_name == 'Üser'

    def test_long_password(self) -> None:
        long_password = 'a' * 100
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password=long_password
        )
        assert user.check_password(long_password) is True

    def test_special_characters_in_email(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test+special@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.email == 'test+special@example.com'

    def test_wrong_password_fails(self) -> None:
        user = register_new_user(
            first_name='Test',
            last_name='User',
            email='test@example.com',
            password='securepassword123'  # noqa: S106
        )
        assert user.check_password('wrongpassword') is False



from __future__ import annotations

import json

from typing import TYPE_CHECKING

from django import forms

from django_spire.file import widgets
from django_spire.file.queryset import FileQuerySet

if TYPE_CHECKING:
    from django_spire.file.models import File


class MultipleFileField(forms.FileField):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.widget = widgets.MultipleWidget()

    def prepare_value(self, value: list[File] | None) -> str:
        if value is not None:
            return json.dumps([file.to_dict() for file in value])

        return json.dumps([])

    def clean(self, data, initial=None) -> dict:
        return data


class SingleFileField(forms.FileField):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.widget = widgets.SingleFileWidget()

    def prepare_value(self, value: File | FileQuerySet | None) -> str:
        if isinstance(value, FileQuerySet):
            value = value.first()

        if value is not None:
            return value.to_json()

        return json.dumps(None)

    def clean(self, data, initial=None) -> dict:
        return data



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.models import HelpDeskTicket
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


class HelpDeskTicketQuerySetTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.ticket1 = create_test_helpdesk_ticket(description='Ticket 1')
        self.ticket2 = create_test_helpdesk_ticket(description='Ticket 2')

    def test_active_returns_non_deleted(self):
        result = HelpDeskTicket.objects.active()
        assert self.ticket1 in result
        assert self.ticket2 in result

    def test_active_excludes_deleted(self):
        self.ticket1.is_deleted = True
        self.ticket1.save()

        result = HelpDeskTicket.objects.active()
        assert self.ticket1 not in result
        assert self.ticket2 in result

    def test_active_excludes_inactive(self):
        self.ticket1.is_active = False
        self.ticket1.save()

        result = HelpDeskTicket.objects.active()
        assert self.ticket1 not in result
        assert self.ticket2 in result

    def test_order_by_created_datetime(self):
        result = list(HelpDeskTicket.objects.order_by('-created_datetime'))
        assert result[0].created_datetime >= result[1].created_datetime



from __future__ import annotations

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.help_desk.tests.factories import create_test_helpdesk_ticket


class HelpDeskPageViewsTestCase(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.test_ticket = create_test_helpdesk_ticket()

    def test_ticket_delete_view(self):
        response = self.client.post(
            reverse(
                'django_spire:help_desk:page:delete',
                kwargs={'pk': self.test_ticket.pk}
            ),
            data={'should_delete': 'on'},
        )

        assert response.status_code == 302
        self.test_ticket.refresh_from_db()
        assert self.test_ticket.is_deleted is True

    def test_ticket_detail_view(self):
        response = self.client.get(
            reverse(
                'django_spire:help_desk:page:detail',
                kwargs={'pk': self.test_ticket.pk}
            ),
        )

        assert response.status_code == 200
        assert response.context_data['ticket'] == self.test_ticket

    def test_ticket_list_view(self):
        tickets = [create_test_helpdesk_ticket() for _ in range(5)]

        response = self.client.get(
            reverse('django_spire:help_desk:page:list'),
        )

        assert response.status_code == 200
        for ticket in tickets:
            assert ticket in response.context_data['tickets']

    def test_ticket_list_view_excludes_deleted(self):
        deleted_ticket = create_test_helpdesk_ticket()
        deleted_ticket.is_deleted = True
        deleted_ticket.save()

        response = self.client.get(
            reverse('django_spire:help_desk:page:list'),
        )

        assert deleted_ticket not in response.context_data['tickets']

    def test_ticket_list_view_ordered_by_created_datetime_desc(self):
        response = self.client.get(
            reverse('django_spire:help_desk:page:list'),
        )

        tickets = list(response.context_data['tickets'])
        if len(tickets) > 1:
            for i in range(len(tickets) - 1):
                assert tickets[i].created_datetime >= tickets[i + 1].created_datetime



from __future__ import annotations

import json

from typing import TYPE_CHECKING

from django.shortcuts import get_object_or_404
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib.generic_views import portal_views
from django_spire.knowledge.collection.models import Collection
from django_spire.knowledge.entry.version.models import EntryVersion

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest
    from django.template.response import TemplateResponse

    from django_spire.contrib.breadcrumb.breadcrumbs import Breadcrumbs


@AppAuthController('knowledge').permission_required('can_view')
def editor_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    entry_version = get_object_or_404(EntryVersion.objects.prefetch_blocks(),pk=pk)

    entry = entry_version.entry
    top_level_collection = entry.top_level_collection
    version_blocks = entry_version.blocks.format_for_editor()

    def breadcrumbs_func(breadcrumbs: Breadcrumbs):
        breadcrumbs.add_breadcrumb(
            name='Knowledge',
            href=reverse('django_spire:knowledge:page:home')
        )

        breadcrumbs.add_base_breadcrumb(entry)

    return portal_views.detail_view(
        request,
        obj=entry,
        breadcrumbs_func=breadcrumbs_func,
        context_data={
            'entry': entry,
            'current_version': entry_version,
            'collection': top_level_collection,
            'version_blocks': json.dumps(list(version_blocks)),
            'collection_tree_json': Collection.services.transformation.to_hierarchy_json(
                request=request,
                parent_id=top_level_collection.id,
            ),
        },
        template='django_spire/knowledge/entry/version/page/editor_page.html',
    )



from __future__ import annotations

from unittest.mock import MagicMock, patch

from django.test import RequestFactory, TestCase, override_settings

from django_spire.core.context_processors import django_spire, theme_context


class TestDjangoSpireContextProcessor(TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.factory = RequestFactory()

    @override_settings(DJANGO_SPIRE_AUTH_CONTROLLERS=[])
    def test_empty_auth_controllers(self) -> None:
        request = self.factory.get('/')
        result = django_spire(request)

        assert 'DJANGO_SPIRE_VERSION' in result
        assert 'AuthController' in result
        assert result['AuthController'] == {}

    @override_settings(DJANGO_SPIRE_AUTH_CONTROLLERS=['test_app'])
    @patch('django_spire.core.context_processors.AppAuthController')
    def test_with_auth_controllers(self, mock_controller: MagicMock) -> None:
        mock_instance = MagicMock()
        mock_controller.return_value = mock_instance

        request = self.factory.get('/')
        result = django_spire(request)

        assert 'AuthController' in result
        assert 'test_app' in result['AuthController']
        mock_controller.assert_called_once_with('test_app', request=request)


class TestThemeContextProcessor(TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.factory = RequestFactory()

    def test_default_theme_applied(self) -> None:
        request = self.factory.get('/')
        request.COOKIES = {}
        result = theme_context(request)

        assert 'DJANGO_SPIRE_DEFAULT_THEME' in result
        assert 'DJANGO_SPIRE_THEME_COOKIE_NAME' in result
        assert 'DJANGO_SPIRE_THEME_PATH' in result
        assert 'theme' in result

    @override_settings(DJANGO_SPIRE_DEFAULT_THEME='default-dark')
    def test_custom_default_theme(self) -> None:
        request = self.factory.get('/')
        request.COOKIES = {}
        result = theme_context(request)

        assert result['DJANGO_SPIRE_DEFAULT_THEME'] == 'default-dark'

    @override_settings(DJANGO_SPIRE_THEME_PATH='/custom/path/{family}/app-{mode}.css')
    def test_custom_theme_path(self) -> None:
        request = self.factory.get('/')
        request.COOKIES = {}
        result = theme_context(request)

        assert result['DJANGO_SPIRE_THEME_PATH'] == '/custom/path/{family}/app-{mode}.css'

    @patch('django_spire.core.context_processors.get_theme_cookie_name')
    @patch('django_spire.core.context_processors.Theme')
    def test_theme_from_cookie(self, mock_theme: MagicMock, mock_cookie_name: MagicMock) -> None:
        mock_cookie_name.return_value = 'theme_cookie'
        mock_theme_instance = MagicMock()
        mock_theme_instance.value = 'custom-dark'
        mock_theme_instance.to_dict.return_value = {'family': 'custom', 'mode': 'dark'}
        mock_theme.from_string.return_value = mock_theme_instance
        mock_theme.get_default.return_value = mock_theme_instance

        request = self.factory.get('/')
        request.COOKIES = {'theme_cookie': 'custom-dark'}
        result = theme_context(request)

        assert result['theme'] == {'family': 'custom', 'mode': 'dark'}

    def test_theme_result_contains_required_keys(self) -> None:
        request = self.factory.get('/')
        request.COOKIES = {}
        result = theme_context(request)

        assert 'theme' in result
        assert isinstance(result['theme'], dict)



from __future__ import annotations

from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType

from django_spire.auth.user.models import AuthUser
from django_spire.comment.models import Comment
from django_spire.comment.utils import (
    find_user_list_from_content_type,
    generate_comment_user_list_data,
    parse_user_id_to_int_list,
)
from django_spire.core.tests.test_cases import BaseTestCase


class TestGenerateCommentUserListData(BaseTestCase):
    def test_single_user(self) -> None:
        self.super_user.first_name = 'Test'
        self.super_user.last_name = 'User'
        self.super_user.save()
        user_list = [self.super_user]
        data = generate_comment_user_list_data(user_list)
        assert len(data) == 1
        assert data[0]['full_name'] == 'Test_User'
        assert data[0]['id'] == self.super_user.pk

    def test_empty_list(self) -> None:
        data = generate_comment_user_list_data([])
        assert data == []

    def test_multiple_users(self) -> None:
        self.super_user.first_name = 'Test'
        self.super_user.last_name = 'User'
        self.super_user.save()
        other_user = AuthUser.objects.create_user(
            username='other',
            first_name='Other',
            last_name='User'
        )
        user_list = [self.super_user, other_user]
        data = generate_comment_user_list_data(user_list)
        assert len(data) == 2
        full_names = [d['full_name'] for d in data]
        assert 'Test_User' in full_names
        assert 'Other_User' in full_names


class TestParseUserIdToIntList(BaseTestCase):
    def test_multiple_ids(self) -> None:
        other_user = AuthUser.objects.create_user(username='other')
        user_id_str = f'{self.super_user.pk},{other_user.pk}'
        users = parse_user_id_to_int_list(user_id_str)
        assert self.super_user in users
        assert other_user in users

    def test_single_id(self) -> None:
        user_id_str = str(self.super_user.pk)
        users = parse_user_id_to_int_list(user_id_str)
        assert self.super_user in users
        assert users.count() == 1


class TestFindUserListFromContentType(BaseTestCase):
    def test_find_users_with_permission(self) -> None:
        content_type = ContentType.objects.get_for_model(Comment)
        permission = Permission.objects.get(
            content_type=content_type,
            codename='view_comment'
        )
        group = Group.objects.create(name='test_group')
        group.permissions.add(permission)
        self.super_user.groups.add(group)

        users = find_user_list_from_content_type('django_spire_comment', 'comment')
        assert self.super_user in users

    def test_excludes_users_without_permission(self) -> None:
        other_user = AuthUser.objects.create_user(username='other')
        content_type = ContentType.objects.get_for_model(Comment)
        permission = Permission.objects.get(
            content_type=content_type,
            codename='view_comment'
        )
        group = Group.objects.create(name='test_group')
        group.permissions.add(permission)
        self.super_user.groups.add(group)

        users = find_user_list_from_content_type('django_spire_comment', 'comment')
        assert self.super_user in users
        assert other_user not in users



from __future__ import annotations

from abc import ABC
from typing import Any, Generic, TypeVar

from django_spire.contrib.constructor.exceptions import ConstructorError


TypeAny = TypeVar('TypeAny', bound=Any, covariant=True)


class BaseConstructor(
    ABC,
    Generic[TypeAny]
):
    def __init__(self, obj: Any = None):
        self._obj_type_name: str = str(
            next(iter(self.__class__.__annotations__.values()))
        ).split('.')[-1]

        if obj is None:
            return

        self._obj_mro_type_names = [cls.__name__ for cls in obj.__class__.__mro__]

        if self._obj_type_name not in self._obj_mro_type_names:
            message = f'{self.__class__.__name__} was instantiated with obj type "{obj.__class__.__name__}" and failed as it was expecting "{self._obj_type_name}".'
            raise ConstructorError(message)

        self._obj_type: type[TypeAny] = obj.__class__

        if self._obj_type is None or self._obj_type is ...:
            message = f'{self.__class__.__name__} top class attribute must have an annotated type.'
            raise ConstructorError(message)

        self.obj: TypeAny = obj

        if ABC not in self.__class__.__bases__:
            if not self._obj_is_valid:
                message = f'{self._obj_type_name} failed to validate on {self.__class__.__name__}'
                raise ConstructorError(message)

        self.__post_init__()

    def __init_subclass__(cls):
        super().__init_subclass__()

        if ABC not in cls.__bases__:
            if 'obj' not in cls.__annotations__:
                message = f'{cls.__name__} must have an "obj" attribute annotated with a type.'
                raise ConstructorError(message)

            # Typing Does not work properly for services if you override __get__ in the BaseService class.
            # This is a workaround and should be fixed in future versions of the python lsp.
            def __get__(self, instance, owner):

                if instance is None:
                    target: cls | Any = owner()
                else:
                    target: cls | Any = instance

                if issubclass(target.__class__, BaseConstructor):

                    self._validate_base_service_target_or_error(target)

                    return cls(target.obj)

                return cls(target)

            cls.__get__ = __get__

    def __post_init__(self):
        pass

    @property
    def obj_class(self) -> type[TypeAny]:
        return self._obj_type

    @property
    def _obj_is_valid(self) -> bool:
        return isinstance(self.obj, self._obj_type)

    def _validate_base_service_target_or_error(self, target: BaseConstructor):
        if self._obj_type_name not in target._obj_mro_type_names:
            message = f'{target.__class__.__name__} must use the same obj type as {self.__class__.__name__}. {self._obj_type_name} is not in {target._obj_mro_type_names}'
            raise ConstructorError(message)



from __future__ import annotations

import enum

from typing import get_args, get_origin, Union
from unittest import TestCase

from pydantic import BaseModel

from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models

from django_spire.core.converters.to_pydantic import DjangoToPydanticFieldConverter, django_to_pydantic_model


def is_optional(t: type) -> bool:
    return get_origin(t) is Union and type(None) in get_args(t)


class TestDjangoModelToPydanticModel(TestCase):
    def test_comprehensive_model_conversion(self) -> None:
        class ComprehensiveModel(models.Model):
            char_field_with_choices = models.CharField(
                max_length=10,
                choices=[('A', 'Active'), ('I', 'Inactive')],
                default='A',
                help_text='Status of the item',
                unique=True,
                null=False
            )
            char_field_no_choices = models.CharField(
                max_length=20,
                default='default',
                help_text='Description',
                unique=False,
                null=True
            )
            int_field = models.IntegerField(
                validators=[MinValueValidator(0), MaxValueValidator(100)],
                default=50,
                help_text='Count',
                unique=False,
                null=True
            )
            decimal_field = models.DecimalField(
                max_digits=5,
                decimal_places=2,
                default=0.0,
                help_text='Price',
                unique=False,
                null=False
            )
            boolean_field = models.BooleanField(
                default=True,
                help_text='Active',
                null=False
            )

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        PydanticModel = django_to_pydantic_model(ComprehensiveModel)
        assert PydanticModel.__name__ == 'ComprehensiveModel'

    def test_conversion_with_custom_base_class(self) -> None:
        class CustomBase(BaseModel):
            extra_field: str = 'extra'

        class CustomModel(models.Model):
            char_field = models.CharField(max_length=10, default='custom')

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        PydanticModel = django_to_pydantic_model(CustomModel, base_class=CustomBase)
        instance = PydanticModel(id=1, char_field='custom', extra_field='extra')

        assert instance.char_field == 'custom'
        assert instance.extra_field == 'extra'

    def test_exclude_fields(self) -> None:
        class SimpleModel2(models.Model):
            field1 = models.CharField(max_length=50, default='value1')
            field2 = models.IntegerField(default=10)
            field3 = models.BooleanField(default=True)

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        PydanticModel = django_to_pydantic_model(SimpleModel2, exclude_fields=['field2'])
        fields = PydanticModel.model_fields
        assert 'id' in fields
        assert 'field1' in fields
        assert 'field3' in fields
        assert 'field2' not in fields

    def test_include_and_exclude(self) -> None:
        class SimpleModel3(models.Model):
            field1 = models.CharField(max_length=50, default='value1')
            field2 = models.IntegerField(default=10)
            field3 = models.BooleanField(default=True)
            field4 = models.CharField(max_length=100, default='value4')

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        PydanticModel = django_to_pydantic_model(
            SimpleModel3,
            include_fields=['id', 'field1', 'field2', 'field3'],
            exclude_fields=['field2']
        )

        fields = PydanticModel.model_fields
        assert 'id' in fields
        assert 'field1' in fields
        assert 'field3' in fields
        assert 'field2' not in fields
        assert 'field4' not in fields

    def test_include_fields_only(self) -> None:
        class SimpleModel1(models.Model):
            field1 = models.CharField(max_length=50, default='value1')
            field2 = models.IntegerField(default=10)
            field3 = models.BooleanField(default=True)

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        PydanticModel = django_to_pydantic_model(SimpleModel1, include_fields=['id', 'field1', 'field3'])
        fields = PydanticModel.model_fields
        assert 'id' in fields
        assert 'field1' in fields
        assert 'field3' in fields
        assert 'field2' not in fields


class TestDjangoToPydanticFieldConverter(TestCase):
    # def test_nullable_field_is_optional(self):
    #     class Model(models.Model):
    #         char_field = models.CharField(max_length=10, null=True)

    #         class Meta:
    #             managed = False

    #         def __str__(self) -> str:
    #             return 'model'

    #     field = Model._meta.get_field('char_field')
    #     field_type, _ = DjangoToPydanticFieldConverter(field).build_field()

    #     assert is_optional(field_type)

    def test_charfield_constraints_applied(self) -> None:
        class Model4(models.Model):
            name = models.CharField(max_length=30)

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        field = Model4._meta.get_field('name')
        field_type, _ = DjangoToPydanticFieldConverter(field).build_field()

        assert 'max_length=30' in str(field_type)

    def test_decimalfield_constraints_applied(self) -> None:
        class Model6(models.Model):
            amount = models.DecimalField(max_digits=5, decimal_places=2)

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        field = Model6._meta.get_field('amount')
        field_type, _ = DjangoToPydanticFieldConverter(field).build_field()

        assert 'max_digits=5' in str(field_type)
        assert 'decimal_places=2' in str(field_type)

    def test_enum_created_for_choices(self) -> None:
        class Model3(models.Model):
            status = models.CharField(
                max_length=10,
                choices=[('A', 'Active'), ('I', 'Inactive')],
                default='A'
            )

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        field = Model3._meta.get_field('status')
        field_type, _ = DjangoToPydanticFieldConverter(field).build_field()

        assert issubclass(field_type, enum.Enum)
        assert set(field_type.__members__) == {'A', 'I'}

    def test_field_with_no_default_sets_none(self) -> None:
        class Model1(models.Model):
            char_field = models.CharField(max_length=10, null=True)

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        field = Model1._meta.get_field('char_field')
        _, field_info = DjangoToPydanticFieldConverter(field).build_field()

        assert field_info.default is None

    def test_help_text_becomes_description(self) -> None:
        class Model2(models.Model):
            char_field = models.CharField(max_length=10, help_text='hello')

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        field = Model2._meta.get_field('char_field')
        _, field_info = DjangoToPydanticFieldConverter(field).build_field()

        assert field_info.description == 'hello'

    def test_integerfield_validators_become_constraints(self) -> None:
        # TODO(Nathan): test needs to be corrected.

        class Model5(models.Model):
            count = models.IntegerField(validators=[MinValueValidator(1), MaxValueValidator(10)])

            class Meta:
                managed = False

            def __str__(self) -> str:
                return 'model'

        field = Model5._meta.get_field('count')
        field_type, _ = DjangoToPydanticFieldConverter(field).build_field()
        assert 'int' in str(field_type)



from __future__ import annotations

from dandy import Bot, LlmConfigOptions, Prompt

from django_spire.ai.prompt.tuning import prompts, intel


class PromptTestingBot(Bot):
    llm_role = Prompt()
    llm_config_options = LlmConfigOptions(temperature=0.4)

    def process(
        self,
        system_prompt: str,
        user_prompt: str
    ) -> intel.PromptTestingIntel:
        self.llm_role = system_prompt
        return self.llm.prompt_to_intel(
            prompt=user_prompt,
            intel_class=intel.PromptTestingIntel
        )


class SimplePromptTuningBot(Bot):
    llm_role = prompts.prompt_tuning_instruction_bot_prompt()
    llm_config_options = LlmConfigOptions(temperature=0.1)

    def process(
        self,
        prompt: str,
        feedback: str
    ) -> intel.PromptTuningIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.prompt_tuning_input_prompt(prompt, feedback),
            intel_class=intel.PromptTuningIntel
        )


class AdvancedPromptTuningBot(Bot):
    llm_role = prompts.prompt_tuning_instruction_bot_prompt()

    def process(
        self,
        system_prompt: str,
        feedback: str
    ) -> intel.PromptTuningIntel:
        simple_bot = SimplePromptTuningBot()
        formatting_bot = FormattingBot()
        duplication_bot = DuplicationRemovalBot()
        instruction_bot = InstructionClarityBot()
        example_bot = ExampleOptimizationBot()
        persona_bot = PersonaBot()

        tuned_prompt = simple_bot.process(system_prompt, feedback)
        formatted_prompt = formatting_bot.process(tuned_prompt.prompt)
        remove_duplicates = duplication_bot.process(formatted_prompt.prompt)
        improve_instructions = instruction_bot.process(remove_duplicates.prompt)
        example_optimization = example_bot.process(improve_instructions.prompt)
        return persona_bot.process(example_optimization.prompt)


class FormattingBot(Bot):
    llm_role = prompts.formatting_bot_instruction_prompt()

    def process(self, system_prompt: str) -> intel.PromptTuningIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            intel_class=intel.PromptTuningIntel
        )


class InstructionClarityBot(Bot):
    llm_role = prompts.instruction_clarity_bot_instruction_prompt()

    def process(self, system_prompt: str) -> intel.PromptTuningIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            intel_class=intel.PromptTuningIntel
        )


class PersonaBot(Bot):
    llm_role = prompts.persona_bot_instruction_prompt()

    def process(self, system_prompt: str) -> intel.PromptTuningIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            intel_class=intel.PromptTuningIntel
        )


class DuplicationRemovalBot(Bot):
    llm_role = prompts.duplication_removal_bot_instruction_prompt()

    def process(self, system_prompt: str) -> intel.PromptTuningIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            intel_class=intel.PromptTuningIntel
        )


class ExampleOptimizationBot(Bot):
    llm_role = prompts.example_optimization_bot_instruction_prompt()

    def process(self, system_prompt: str) -> intel.PromptTuningIntel:
        return self.llm.prompt_to_intel(
            prompt=prompts.specialized_bot_input_prompt(system_prompt),
            intel_class=intel.PromptTuningIntel
        )



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from django.contrib.auth.models import User
    from django.db.models import Model


def create_or_update_comment(obj: Model, user: User, information: str):
    pass



from __future__ import annotations

from django_spire.auth.group.models import AuthGroup
from django_spire.auth.user.models import AuthUser
from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase


class AuthUserServiceTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user1 = create_user(
            username='user1',
            first_name='Alice',
            last_name='Smith',
            is_active=True
        )
        self.user2 = create_user(
            username='user2',
            first_name='Bob',
            last_name='Jones',
            is_active=True
        )
        self.inactive_user = create_user(
            username='inactive',
            first_name='Charlie',
            last_name='Brown',
            is_active=False
        )

    def test_get_user_choices_returns_list(self) -> None:
        choices = AuthUser.services.get_user_choices()
        assert isinstance(choices, list)

    def test_get_user_choices_format(self) -> None:
        choices = AuthUser.services.get_user_choices()
        for choice in choices:
            assert len(choice) == 2
            assert isinstance(choice[0], int)
            assert isinstance(choice[1], str)

    def test_get_user_choices_excludes_inactive(self) -> None:
        choices = AuthUser.services.get_user_choices()
        user_ids = [c[0] for c in choices]
        assert self.inactive_user.pk not in user_ids

    def test_get_user_choices_includes_active(self) -> None:
        choices = AuthUser.services.get_user_choices()
        user_ids = [c[0] for c in choices]
        assert self.user1.pk in user_ids
        assert self.user2.pk in user_ids

    def test_get_user_choices_by_group(self) -> None:
        group = AuthGroup.objects.create(name='Test Group')
        self.user1.groups.add(group)

        choices = AuthUser.services.get_user_choices_by_group(group)
        user_ids = [c[0] for c in choices]

        assert self.user1.pk in user_ids
        assert self.user2.pk not in user_ids

    def test_get_user_choices_by_group_empty(self) -> None:
        group = AuthGroup.objects.create(name='Empty Group')
        choices = AuthUser.services.get_user_choices_by_group(group)
        assert len(choices) == 0

    def test_get_user_choices_by_group_multiple_users(self) -> None:
        group = AuthGroup.objects.create(name='Multi User Group')
        self.user1.groups.add(group)
        self.user2.groups.add(group)

        choices = AuthUser.services.get_user_choices_by_group(group)
        user_ids = [c[0] for c in choices]

        assert self.user1.pk in user_ids
        assert self.user2.pk in user_ids

    def test_get_user_choices_by_group_excludes_inactive(self) -> None:
        group = AuthGroup.objects.create(name='Mixed Group')
        self.user1.groups.add(group)
        self.inactive_user.groups.add(group)

        choices = AuthUser.services.get_user_choices_by_group(group)
        user_ids = [c[0] for c in choices]

        assert self.user1.pk in user_ids
        assert self.inactive_user.pk not in user_ids

    def test_get_user_choices_returns_full_name(self) -> None:
        choices = AuthUser.services.get_user_choices()
        user1_choice = next((c for c in choices if c[0] == self.user1.pk), None)
        assert user1_choice is not None
        assert user1_choice[1] == 'Alice Smith'

    def test_get_user_choices_with_no_active_users(self) -> None:
        AuthUser.objects.filter(is_active=True).update(is_active=False)
        choices = AuthUser.services.get_user_choices()
        assert len(choices) == 0

    def test_get_user_choices_user_in_multiple_groups(self) -> None:
        group1 = AuthGroup.objects.create(name='Group 1')
        group2 = AuthGroup.objects.create(name='Group 2')
        self.user1.groups.add(group1, group2)

        choices1 = AuthUser.services.get_user_choices_by_group(group1)
        choices2 = AuthUser.services.get_user_choices_by_group(group2)

        assert any(c[0] == self.user1.pk for c in choices1)
        assert any(c[0] == self.user1.pk for c in choices2)

    def test_get_user_choices_empty_name(self) -> None:
        user = create_user(username='emptyname', first_name='', last_name='')
        choices = AuthUser.services.get_user_choices()
        user_choice = next((c for c in choices if c[0] == user.pk), None)
        assert user_choice is not None
        assert user_choice[1] == ''

    def test_get_user_choices_unicode_name(self) -> None:
        user = create_user(username='unicode', first_name='Tëst', last_name='Üsér')
        choices = AuthUser.services.get_user_choices()
        user_choice = next((c for c in choices if c[0] == user.pk), None)
        assert user_choice is not None
        assert 'Tëst' in user_choice[1]
        assert 'Üsér' in user_choice[1]

    def test_get_user_choices_first_name_only(self) -> None:
        user = create_user(username='firstonly', first_name='First', last_name='')
        choices = AuthUser.services.get_user_choices()
        user_choice = next((c for c in choices if c[0] == user.pk), None)
        assert user_choice is not None
        assert 'First' in user_choice[1]

    def test_get_user_choices_last_name_only(self) -> None:
        user = create_user(username='lastonly', first_name='', last_name='Last')
        choices = AuthUser.services.get_user_choices()
        user_choice = next((c for c in choices if c[0] == user.pk), None)
        assert user_choice is not None
        assert 'Last' in user_choice[1]

    def test_get_user_choices_by_group_nonexistent_group(self) -> None:
        group = AuthGroup.objects.create(name='Temp Group')
        group_pk = group.pk
        group.delete()
        new_group = AuthGroup(pk=group_pk, name='Fake')
        choices = AuthUser.services.get_user_choices_by_group(new_group)
        assert len(choices) == 0

    def test_get_user_choices_many_users(self) -> None:
        for i in range(50):
            create_user(username=f'manyuser{i}', first_name=f'User{i}', last_name='Test')
        choices = AuthUser.services.get_user_choices()
        assert len(choices) >= 50

    def test_get_user_choices_by_group_after_user_removal(self) -> None:
        group = AuthGroup.objects.create(name='Removal Group')
        self.user1.groups.add(group)

        choices_before = AuthUser.services.get_user_choices_by_group(group)
        assert any(c[0] == self.user1.pk for c in choices_before)

        self.user1.groups.remove(group)

        choices_after = AuthUser.services.get_user_choices_by_group(group)
        assert not any(c[0] == self.user1.pk for c in choices_after)

    def test_get_user_choices_by_group_after_user_deactivation(self) -> None:
        group = AuthGroup.objects.create(name='Deactivation Group')
        self.user1.groups.add(group)

        choices_before = AuthUser.services.get_user_choices_by_group(group)
        assert any(c[0] == self.user1.pk for c in choices_before)

        self.user1.is_active = False
        self.user1.save()

        choices_after = AuthUser.services.get_user_choices_by_group(group)
        assert not any(c[0] == self.user1.pk for c in choices_after)

    def test_service_obj_class_is_auth_user(self) -> None:
        assert AuthUser.services.obj_class == AuthUser

    def test_get_user_choices_order(self) -> None:
        choices = AuthUser.services.get_user_choices()
        assert len(choices) > 0

    def test_get_user_choices_by_group_returns_list(self) -> None:
        group = AuthGroup.objects.create(name='List Group')
        choices = AuthUser.services.get_user_choices_by_group(group)
        assert isinstance(choices, list)

    def test_get_user_choices_by_group_format(self) -> None:
        group = AuthGroup.objects.create(name='Format Group')
        self.user1.groups.add(group)
        choices = AuthUser.services.get_user_choices_by_group(group)
        for choice in choices:
            assert len(choice) == 2
            assert isinstance(choice[0], int)
            assert isinstance(choice[1], str)



from __future__ import annotations

from typing import TYPE_CHECKING

from django import template

if TYPE_CHECKING:
    from typing import Any


register = template.Library()


@register.filter
def is_dict(value: Any) -> bool:
    return isinstance(value, dict)


@register.filter
def is_not_dict(value: Any) -> bool:
    return not is_dict(value)


@register.filter
def is_list(value: Any) -> bool:
    return isinstance(value, list)


@register.filter
def is_not_list(value: Any) -> bool:
    return not is_list(value)


@register.filter
def is_list_or_tuple(value: Any) -> bool:
    return isinstance(value, (list, tuple))


@register.filter
def is_not_list_or_tuple(value: Any) -> bool:
    return not is_list_or_tuple(value)


@register.filter
def is_tuple(value: Any) -> bool:
    return isinstance(value, tuple)


@register.filter
def is_not_tuple(value: Any) -> bool:
    return not is_tuple(value)



from __future__ import annotations

from django.contrib.auth.models import User

from django_spire.notification.choices import (
    NotificationStatusChoices,
    NotificationTypeChoices,
)
from django_spire.notification.models import Notification
from django_spire.notification.sms.models import SmsNotification


def create_test_sms_notification(**kwargs) -> SmsNotification:
    if 'notification' not in kwargs:
        user = kwargs.pop('user', None) or User.objects.first()
        notification = Notification.objects.create(
            user=user,
            type=NotificationTypeChoices.SMS,
            title=kwargs.pop('title', 'Test SMS Notification'),
            body=kwargs.pop('body', 'This is a test SMS notification.'),
            url=kwargs.pop('url', ''),
            status=kwargs.pop('status', NotificationStatusChoices.PENDING),
            priority=kwargs.pop('priority', 'low'),
        )
        kwargs['notification'] = notification

    data = {
        'to_phone_number': '5551234567',
        'media_url': None,
        'temporary_media': None,
    }
    data.update(kwargs)
    return SmsNotification.objects.create(**data)



from __future__ import annotations

from functools import wraps

from typing import Callable, ParamSpec, TypeVar

from django.db import connections
from django.http import HttpRequest, HttpResponse, JsonResponse


P = ParamSpec('P')
T = TypeVar('T')


def close_db_connections(func: Callable[P, T]) -> Callable[P, T]:
    @wraps(func)
    def inner(*args: P.args, **kwargs: P.kwargs) -> T:
        try:
            return func(*args, **kwargs)
        finally:
            connections.close_all()

    return inner


def valid_ajax_request_required(
    method: Callable[..., HttpResponse]
) -> Callable[..., HttpResponse]:
    @wraps(method)
    def wrapper(request: HttpRequest, *args, **kwargs) -> HttpResponse:
        if request.method != 'POST' and request.content_type != 'application/json':
            return JsonResponse(
                {'type': 'error', 'message': 'Invalid Request'}
            )

        return method(request, *args, **kwargs)

    return wrapper



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.contrib.seeding.field.base import BaseFieldSeeder

if TYPE_CHECKING:
    from typing import Any


class StaticFieldSeeder(BaseFieldSeeder):
    keyword = 'static'

    def seed(self, manager: Any = None, count: int = 1) -> list[dict]:
        return [
            {field_name: value[1] for field_name, value in self.seeder_fields.items()}
            for _ in range(count)
        ]



from __future__ import annotations

from collections import defaultdict

from django_spire.notification.app.processor import AppNotificationProcessor
from django_spire.notification.choices import (
    NotificationStatusChoices,
    NotificationTypeChoices
)
from django_spire.notification.email.processor import EmailNotificationProcessor
from django_spire.notification.exceptions import NotificationError
from django_spire.notification.models import Notification
from django_spire.notification.processors.processor import BaseNotificationProcessor
from django_spire.notification.sms.processor import SMSNotificationProcessor


class NotificationProcessor(BaseNotificationProcessor):
    def process(self, notification: Notification):
        processor = self._get_processor(notification.type)

        if processor is None:
            notification.status = NotificationStatusChoices.FAILED
            notification.save()

            message = f'Unknown notification type: {notification.type}'
            raise NotificationError(message)

        processor().process(notification)

    def process_list(self, notifications: list[Notification]):
        sorted_notifications = defaultdict(list)

        for notification in notifications:
            sorted_notifications[notification.type].append(notification)

        for notification_type, notifications in sorted_notifications.items():
            processor = self._get_processor(notification_type)

            if processor is None:
                for notification in notifications:
                    notification.status = NotificationStatusChoices.FAILED

                Notification.objects.bulk_update(notifications, ['status'])
                continue

            processor().process_list(notifications)

    @staticmethod
    def _get_processor(
        notification_type: NotificationTypeChoices
    ) -> type[BaseNotificationProcessor] | None:
        if notification_type == NotificationTypeChoices.APP:
            return AppNotificationProcessor

        if notification_type == NotificationTypeChoices.EMAIL:
            return EmailNotificationProcessor

        if notification_type == NotificationTypeChoices.SMS:
            return SMSNotificationProcessor

        return None

    def process_ready(self):
        self.process_list(Notification.objects.ready_to_send().active())

    def process_errored(self):
        self.process_list(Notification.objects.errored().active())



from __future__ import annotations

from django.urls import reverse

from django_spire.auth.group.models import AuthGroup
from django_spire.auth.user.models import AuthUser
from django_spire.auth.user.tests.factories import create_user
from django_spire.core.tests.test_cases import BaseTestCase


class UserPageViewsTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(
            username='testuser',
            first_name='Test',
            last_name='User',
            email='test@example.com'
        )

    def test_list_view_requires_permission(self) -> None:
        normal_user = create_user(username='normaluser')
        self.client.force_login(normal_user)
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        assert response.status_code == 403

    def test_list_view_with_permission(self) -> None:
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        assert response.status_code == 200

    def test_list_view_context_contains_users(self) -> None:
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        assert 'active_user_list' in response.context
        assert 'inactive_user_list' in response.context

    def test_list_view_separates_active_inactive(self) -> None:
        inactive_user = create_user(username='inactiveuser', is_active=False)
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        active_ids = [u.pk for u in response.context['active_user_list']]
        inactive_ids = [u.pk for u in response.context['inactive_user_list']]
        assert inactive_user.pk in inactive_ids
        assert inactive_user.pk not in active_ids

    def test_detail_view_requires_permission(self) -> None:
        normal_user = create_user(username='normaluser')
        self.client.force_login(normal_user)
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 403

    def test_detail_view_with_permission(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 200

    def test_detail_view_context_contains_user(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert 'user' in response.context

    def test_detail_view_context_contains_groups(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert 'group_list' in response.context

    def test_detail_view_context_contains_permissions(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert 'user_perm_data' in response.context

    def test_detail_view_404_for_nonexistent_user(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': 99999})
        )
        assert response.status_code == 404

    def test_list_view_contains_active_user(self) -> None:
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        active_ids = [u.pk for u in response.context['active_user_list']]
        assert self.user.pk in active_ids

    def test_list_view_pagination(self) -> None:
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        assert 'active_user_list' in response.context
        assert hasattr(response.context['active_user_list'], 'paginator')

    def test_detail_view_with_groups(self) -> None:
        group = AuthGroup.objects.create(name='Test Group')
        self.user.groups.add(group)
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 200
        assert group in response.context['group_list']

    def test_detail_view_context_contains_group_permission_data(self) -> None:
        group = AuthGroup.objects.create(name='Test Group')
        self.user.groups.add(group)
        response = self.client.get(
            reverse('django_spire:auth:user:page:detail', kwargs={'pk': self.user.pk})
        )
        assert 'group_list_permission_data' in response.context

    def test_list_view_multiple_users(self) -> None:
        create_user(username='user1', first_name='User', last_name='One')
        create_user(username='user2', first_name='User', last_name='Two')
        response = self.client.get(reverse('django_spire:auth:user:page:list'))
        assert len(response.context['active_user_list']) >= 2


class UserFormViewsTestCase(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.user = create_user(
            username='testuser',
            first_name='Test',
            last_name='User',
            email='test@example.com'
        )

    def test_register_form_view_get(self) -> None:
        response = self.client.get(reverse('django_spire:auth:user:form:register'))
        assert response.status_code == 200

    def test_register_form_view_post_creates_user(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert response.status_code == 302
        assert AuthUser.objects.filter(email='newuser@example.com').exists()

    def test_register_form_view_post_invalid_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'short'
            }
        )
        assert response.status_code == 200
        assert not AuthUser.objects.filter(email='newuser@example.com').exists()

    def test_register_form_view_post_invalid_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'invalid-email',
                'password': 'securepassword123'
            }
        )
        assert response.status_code == 200

    def test_register_form_requires_permission(self) -> None:
        normal_user = create_user(username='normaluser')
        self.client.force_login(normal_user)
        response = self.client.get(reverse('django_spire:auth:user:form:register'))
        assert response.status_code == 403

    def test_update_form_view_get(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:form:update', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 200

    def test_update_form_view_404(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:form:update', kwargs={'pk': 99999})
        )
        assert response.status_code == 404

    def test_update_form_requires_permission(self) -> None:
        normal_user = create_user(username='normaluser')
        self.client.force_login(normal_user)
        response = self.client.get(
            reverse('django_spire:auth:user:form:update', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 403

    def test_group_form_view_get(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 200

    def test_group_form_view_post_updates_groups(self) -> None:
        group = AuthGroup.objects.create(name='Test Group')
        response = self.client.post(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk}),
            data={'group_list': [group.pk]}
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert group in self.user.groups.all()

    def test_group_form_view_post_multiple_groups(self) -> None:
        group1 = AuthGroup.objects.create(name='Group 1')
        group2 = AuthGroup.objects.create(name='Group 2')
        response = self.client.post(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk}),
            data={'group_list': [group1.pk, group2.pk]}
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert group1 in self.user.groups.all()
        assert group2 in self.user.groups.all()

    def test_group_form_view_404(self) -> None:
        response = self.client.get(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': 99999})
        )
        assert response.status_code == 404

    def test_group_form_requires_permission(self) -> None:
        normal_user = create_user(username='normaluser')
        self.client.force_login(normal_user)
        response = self.client.get(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk})
        )
        assert response.status_code == 403

    def test_register_form_view_post_empty_password(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': ''
            }
        )
        assert response.status_code == 200

    def test_register_form_view_post_empty_email(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': '',
                'password': 'securepassword123'
            }
        )
        assert response.status_code == 200

    def test_register_form_view_post_unicode_names(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'Tëst',
                'last_name': 'Üsér',
                'email': 'unicode@example.com',
                'password': 'securepassword123'
            }
        )
        assert response.status_code == 302
        user = AuthUser.objects.get(email='unicode@example.com')
        assert user.first_name == 'Tëst'
        assert user.last_name == 'Üsér'

    def test_group_form_view_post_removes_existing_groups(self) -> None:
        group1 = AuthGroup.objects.create(name='Group 1')
        group2 = AuthGroup.objects.create(name='Group 2')
        self.user.groups.add(group1)

        response = self.client.post(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk}),
            data={'group_list': [group2.pk]}
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert group1 not in self.user.groups.all()
        assert group2 in self.user.groups.all()

    def test_group_form_view_post_invalid_group_id(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk}),
            data={'group_list': [99999]}
        )
        assert response.status_code == 200

    def test_register_redirects_after_success(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert response.status_code == 302
        assert 'list' in response.url or 'user' in response.url

    def test_group_form_redirects_after_success(self) -> None:
        group = AuthGroup.objects.create(name='Test Group')
        response = self.client.post(
            reverse('django_spire:auth:user:form:group_form', kwargs={'pk': self.user.pk}),
            data={'group_list': [group.pk]}
        )
        assert response.status_code == 302
        assert str(self.user.pk) in response.url

    def test_update_form_view_post_updates_user(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:update', kwargs={'pk': self.user.pk}),
            data={
                'first_name': 'Updated',
                'last_name': 'Name',
                'email': 'updated@example.com',
                'is_active': True
            }
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert self.user.first_name == 'Updated'
        assert self.user.last_name == 'Name'
        assert self.user.email == 'updated@example.com'

    def test_update_form_view_post_deactivates_user(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:update', kwargs={'pk': self.user.pk}),
            data={
                'first_name': 'Test',
                'last_name': 'User',
                'email': 'test@example.com',
                'is_active': False
            }
        )
        assert response.status_code == 302
        self.user.refresh_from_db()
        assert not self.user.is_active

    def test_register_form_adds_to_all_users_group(self) -> None:
        response = self.client.post(
            reverse('django_spire:auth:user:form:register'),
            data={
                'first_name': 'New',
                'last_name': 'User',
                'email': 'newuser@example.com',
                'password': 'securepassword123'
            }
        )
        assert response.status_code == 302
        user = AuthUser.objects.get(email='newuser@example.com')
        group_names = [g.name for g in user.groups.all()]
        assert 'All Users' in group_names



from __future__ import annotations

import json

from django.test import override_settings

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.file.tests.factories import create_test_file


STORAGES_OVERRIDE = {
    'default': {
        'BACKEND': 'django.core.files.storage.FileSystemStorage',
    },
    'staticfiles': {
        'BACKEND': 'django.contrib.staticfiles.storage.StaticFilesStorage',
    },
}


@override_settings(STORAGES=STORAGES_OVERRIDE)
class FileModelTests(BaseTestCase):
    def setUp(self):
        super().setUp()

        self.file = create_test_file()

    def test_str(self):
        assert str(self.file) == self.file.name

    def test_default_is_active(self):
        assert self.file.is_active is True

    def test_default_is_deleted(self):
        assert self.file.is_deleted is False

    def test_name_field(self):
        assert self.file.name == 'test_file'

    def test_type_field(self):
        assert self.file.type == 'pdf'

    def test_size_field(self):
        assert self.file.size == '1.5 Mb'

    def test_related_field_default(self):
        assert self.file.related_field is None

    def test_related_field_set(self):
        file = create_test_file(related_field='abc')
        assert file.related_field == 'abc'

    def test_content_type_null(self):
        assert self.file.content_type is None

    def test_object_id_null(self):
        assert self.file.object_id is None

    def test_created_datetime_auto_set(self):
        assert self.file.created_datetime is not None

    def test_to_dict(self):
        result = self.file.to_dict()

        assert result['name'] == self.file.name
        assert result['id'] == self.file.id
        assert 'url' in result

    def test_to_dict_keys(self):
        result = self.file.to_dict()

        assert set(result.keys()) == {'name', 'url', 'id'}

    def test_to_json(self):
        result = self.file.to_json()
        parsed = json.loads(result)

        assert parsed['name'] == self.file.name
        assert parsed['id'] == self.file.id

    def test_file_field_url(self):
        assert self.file.file.url is not None



from __future__ import annotations

from typing import TYPE_CHECKING

from django.db.models import QuerySet
from django.utils.timezone import localtime

if TYPE_CHECKING:
    from django.contrib.auth.models import User

    from django_spire.auth.mfa.models import MfaCode


class MfaCodeQuerySet(QuerySet):
    def valid_code(self, user: User) -> MfaCode | None:
        return self.filter(user=user, expiration_datetime__gt=localtime()).first()



from __future__ import annotations

from django.urls import include, path


app_name = 'help_desk'

urlpatterns = [
    path('page/', include('django_spire.help_desk.urls.page_urls', namespace='page')),
    path('form/', include('django_spire.help_desk.urls.form_urls', namespace='form')),
]



from __future__ import annotations

from django_spire.ai.chat.choices import MessageResponseType
from django_spire.ai.chat.message_intel import DefaultMessageIntel
from django_spire.ai.chat.models import Chat
from django_spire.ai.chat.responses import MessageResponse
from django_spire.core.tests.test_cases import BaseTestCase


class ChatModelTests(BaseTestCase):
    def test_chat_creation(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

        assert chat.name == 'Test Chat'
        assert chat.user == self.super_user

    def test_chat_str_short_name(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Short Name'
        )

        assert str(chat) == 'Short Name'

    def test_chat_str_long_name(self) -> None:
        long_name = 'A' * 100
        chat = Chat.objects.create(
            user=self.super_user,
            name=long_name
        )

        assert len(str(chat)) == 51
        assert str(chat).endswith('...')

    def test_chat_name_shortened_short(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Short'
        )

        assert chat.name_shortened == 'Short'

    def test_chat_name_shortened_long(self) -> None:
        long_name = 'A' * 50
        chat = Chat.objects.create(
            user=self.super_user,
            name=long_name
        )

        assert len(chat.name_shortened) == 27
        assert chat.name_shortened.endswith('...')

    def test_chat_is_empty(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Empty Chat'
        )

        assert chat.is_empty

    def test_chat_is_not_empty(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Chat with messages'
        )

        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        )
        chat.add_message_response(message_response)

        assert not chat.is_empty

    def test_add_message_response(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Test message')
        )
        chat.add_message_response(message_response)

        assert chat.messages.count() == 1

    def test_add_message_response_updates_last_message_datetime(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

        initial_datetime = chat.last_message_datetime

        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Test')
        )
        chat.add_message_response(message_response)
        chat.refresh_from_db()

        assert chat.last_message_datetime >= initial_datetime

    def test_generate_message_history(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

        chat.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Hello')
        ))
        chat.add_message_response(MessageResponse(
            type=MessageResponseType.RESPONSE,
            sender='Assistant',
            message_intel=DefaultMessageIntel(text='Hi there')
        ))

        message_history = chat.generate_message_history()

        assert message_history is not None

    def test_generate_message_history_exclude_last(self) -> None:
        chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

        chat.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Message 1')
        ))
        chat.add_message_response(MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Message 2')
        ))

        message_history = chat.generate_message_history(exclude_last_message=True)

        assert message_history is not None


class ChatMessageModelTests(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

    def test_chat_message_str_short(self) -> None:
        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Short')
        )
        self.chat.add_message_response(message_response)

        message = self.chat.messages.first()

        assert str(message) == 'Short'

    def test_chat_message_str_long(self) -> None:
        long_text = 'A' * 100
        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text=long_text)
        )
        self.chat.add_message_response(message_response)

        message = self.chat.messages.first()

        assert len(str(message)) == 67
        assert str(message).endswith('...')

    def test_chat_message_intel_property(self) -> None:
        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Test intel')
        )
        self.chat.add_message_response(message_response)

        message = self.chat.messages.first()

        assert isinstance(message.intel, DefaultMessageIntel)
        assert message.intel.text == 'Test intel'

    def test_chat_message_role_request(self) -> None:
        message_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Test')
        )
        self.chat.add_message_response(message_response)

        message = self.chat.messages.first()

        assert message.role == 'user'

    def test_chat_message_role_response(self) -> None:
        message_response = MessageResponse(
            type=MessageResponseType.RESPONSE,
            sender='Assistant',
            message_intel=DefaultMessageIntel(text='Test')
        )
        self.chat.add_message_response(message_response)

        message = self.chat.messages.first()

        assert message.role == 'assistant'

    def test_chat_message_to_message_response(self) -> None:
        original_response = MessageResponse(
            type=MessageResponseType.REQUEST,
            sender='User',
            message_intel=DefaultMessageIntel(text='Test')
        )
        self.chat.add_message_response(original_response)

        message = self.chat.messages.first()
        converted_response = message.to_message_response()

        assert isinstance(converted_response, MessageResponse)
        assert converted_response.type == MessageResponseType.REQUEST
        assert converted_response.sender == 'User'


class ChatQuerySetTests(BaseTestCase):
    def test_by_user(self) -> None:
        Chat.objects.create(user=self.super_user, name='User Chat')

        result = Chat.objects.by_user(self.super_user)

        assert result.count() == 1

    def test_search(self) -> None:
        Chat.objects.create(user=self.super_user, name='Python Programming')
        Chat.objects.create(user=self.super_user, name='Django Framework')

        result = Chat.objects.search('Python')

        assert result.count() == 1
        assert result.first().name == 'Python Programming'

    def test_search_case_insensitive(self) -> None:
        Chat.objects.create(user=self.super_user, name='Python Programming')

        result = Chat.objects.search('python')

        assert result.count() == 1


class ChatMessageQuerySetTests(BaseTestCase):
    def setUp(self) -> None:
        super().setUp()

        self.chat = Chat.objects.create(
            user=self.super_user,
            name='Test Chat'
        )

    def test_newest_by_count(self) -> None:
        for i in range(25):
            message_response = MessageResponse(
                type=MessageResponseType.REQUEST,
                sender='User',
                message_intel=DefaultMessageIntel(text=f'Message {i}')
            )
            self.chat.add_message_response(message_response)

        messages = self.chat.messages.newest_by_count(20)

        assert len(messages) == 20

    def test_newest_by_count_reversed(self) -> None:
        for i in range(5):
            message_response = MessageResponse(
                type=MessageResponseType.REQUEST,
                sender='User',
                message_intel=DefaultMessageIntel(text=f'Message {i}')
            )
            self.chat.add_message_response(message_response)

        messages = self.chat.messages.newest_by_count_reversed(5)

        assert len(messages) == 5



# Generated by Django 5.2.1 on 2025-05-28 14:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_notification_email', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailnotification',
            name='bcc',
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='cc',
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='context_data',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='emailnotification',
            name='template_id',
            field=models.CharField(default='', max_length=64),
        ),
    ]



from __future__ import annotations

import datetime

from unittest import TestCase
from uuid import UUID

from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models

from django_spire.core.converters.to_data import DjangoFieldToFakerData, fake_model_field_value


class TestDjangoFieldToFakerData(TestCase):
    def test_binary_field_data(self) -> None:
        class BinaryModel(models.Model):
            data = models.BinaryField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = BinaryModel._meta.get_field('data')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, bytes)
        assert len(result) == 16

    def test_boolean_field_data(self) -> None:
        class BooleanModel(models.Model):
            is_active = models.BooleanField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = BooleanModel._meta.get_field('is_active')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, bool)

    def test_char_field_data(self) -> None:
        class CharModel(models.Model):
            name = models.CharField(max_length=50)

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = CharModel._meta.get_field('name')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)
        assert len(result) <= 50

    def test_char_field_with_choices(self) -> None:
        class ChoicesModel(models.Model):
            status = models.CharField(
                max_length=10,
                choices=[('A', 'Active'), ('I', 'Inactive')]
            )

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = ChoicesModel._meta.get_field('status')
        result = DjangoFieldToFakerData(field).convert()

        assert result in ['A', 'I']

    def test_custom_faker_method(self) -> None:
        class CustomFakerModel(models.Model):
            name = models.CharField(max_length=100)

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = CustomFakerModel._meta.get_field('name')
        result = DjangoFieldToFakerData(field, faker_method=('first_name',)).convert()

        assert isinstance(result, str)

    def test_custom_faker_method_with_kwargs(self) -> None:
        class CustomFakerKwargsModel(models.Model):
            number = models.IntegerField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = CustomFakerKwargsModel._meta.get_field('number')
        result = DjangoFieldToFakerData(
            field,
            faker_method=('random_int', {'min': 100, 'max': 200})
        ).convert()

        assert isinstance(result, int)
        assert 100 <= result <= 200

    def test_date_field_data(self) -> None:
        class DateModel(models.Model):
            created = models.DateField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = DateModel._meta.get_field('created')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, datetime.date)

    def test_date_time_field_data(self) -> None:
        class DateTimeModel(models.Model):
            created_at = models.DateTimeField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = DateTimeModel._meta.get_field('created_at')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, datetime.datetime)

    def test_decimal_field_data(self) -> None:
        class DecimalModel(models.Model):
            price = models.DecimalField(max_digits=10, decimal_places=2)

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = DecimalModel._meta.get_field('price')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, float)

    def test_email_field_data(self) -> None:
        """EmailField.get_internal_type() returns 'CharField', so it uses _char_field_data"""

        class EmailModel(models.Model):
            email = models.EmailField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = EmailModel._meta.get_field('email')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)

    def test_integer_field_data(self) -> None:
        class IntegerModel(models.Model):
            count = models.IntegerField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = IntegerModel._meta.get_field('count')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, int)

    def test_integer_field_with_validators(self) -> None:
        class IntegerValidatorModel(models.Model):
            count = models.IntegerField(
                validators=[MinValueValidator(10), MaxValueValidator(20)]
            )

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = IntegerValidatorModel._meta.get_field('count')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, int)
        assert 10 <= result <= 20

    def test_positive_integer_field_data(self) -> None:
        class PositiveIntegerModel(models.Model):
            count = models.PositiveIntegerField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = PositiveIntegerModel._meta.get_field('count')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, int)
        assert result >= 0

    def test_slug_field_data(self) -> None:
        class SlugModel(models.Model):
            slug = models.SlugField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = SlugModel._meta.get_field('slug')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)

    def test_text_field_data(self) -> None:
        class TextModel(models.Model):
            description = models.TextField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = TextModel._meta.get_field('description')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)

    def test_time_field_data(self) -> None:
        class TimeModel(models.Model):
            start_time = models.TimeField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = TimeModel._meta.get_field('start_time')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)

    def test_url_field_data(self) -> None:
        """URLField.get_internal_type() returns 'CharField', so it uses _char_field_data"""

        class UrlModel(models.Model):
            website = models.URLField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = UrlModel._meta.get_field('website')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)

    def test_uuid_field_data(self) -> None:
        class UuidModel(models.Model):
            uuid = models.UUIDField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        field = UuidModel._meta.get_field('uuid')
        result = DjangoFieldToFakerData(field).convert()

        assert isinstance(result, str)
        UUID(result)


class TestFakeModelFieldValue(TestCase):
    def test_fake_model_field_value(self) -> None:
        class FakeValueModel(models.Model):
            name = models.CharField(max_length=50)

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        result = fake_model_field_value(FakeValueModel, 'name')

        assert isinstance(result, str)
        assert len(result) <= 50

    def test_fake_model_field_value_with_faker_method(self) -> None:
        class FakeValueFakerModel(models.Model):
            email = models.EmailField()

            class Meta:
                managed = False
                app_label = 'test_to_data'

            def __str__(self) -> str:
                return 'model'

        result = fake_model_field_value(FakeValueFakerModel, 'email', faker_method=('email',))

        assert isinstance(result, str)
        assert '@' in result



from django.urls import include, path

app_name = 'entry'

urlpatterns = [
    path('version/', include('django_spire.knowledge.entry.version.urls', namespace='version')),

    path('form/', include('django_spire.knowledge.entry.urls.form_urls', namespace='form')),
    path('json/', include('django_spire.knowledge.entry.urls.json_urls', namespace='json')),
    path('page/', include('django_spire.knowledge.entry.urls.page_urls', namespace='page')),
    path('template/', include('django_spire.knowledge.entry.urls.template_urls', namespace='template')),
]



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.entry.version.block.data.heading_data import HeadingEditorBlockData
from django_spire.knowledge.entry.version.block.data.list.choices import ListEditorBlockDataStyle
from django_spire.knowledge.entry.version.block.data.list.data import ListEditorBlockData
from django_spire.knowledge.entry.version.block.data.text_data import TextEditorBlockData


class TextEditorBlockDataTests(BaseTestCase):
    def test_render_to_text(self):
        data = TextEditorBlockData(text='Hello World')
        result = data.render_to_text()
        assert 'Hello World' in result

    def test_render_to_text_with_html(self):
        data = TextEditorBlockData(text='<b>Bold</b> text')
        result = data.render_to_text()
        assert 'Bold' in result


class HeadingEditorBlockDataTests(BaseTestCase):
    def test_render_to_text_h1(self):
        data = HeadingEditorBlockData(text='Title', level=1)
        result = data.render_to_text()
        assert '# Title' in result

    def test_render_to_text_h2(self):
        data = HeadingEditorBlockData(text='Subtitle', level=2)
        result = data.render_to_text()
        assert '## Subtitle' in result

    def test_render_to_text_h3(self):
        data = HeadingEditorBlockData(text='Section', level=3)
        result = data.render_to_text()
        assert '### Section' in result


class ListEditorBlockDataTests(BaseTestCase):
    def test_render_to_text_unordered(self):
        data = ListEditorBlockData(
            style=ListEditorBlockDataStyle.UNORDERED,
            items=[
                {'content': 'Item 1', 'items': []},
                {'content': 'Item 2', 'items': []},
            ]
        )
        result = data.render_to_text()
        assert '- Item 1' in result
        assert '- Item 2' in result

    def test_render_to_text_ordered(self):
        data = ListEditorBlockData(
            style=ListEditorBlockDataStyle.ORDERED,
            meta={'start': 1},
            items=[
                {'content': 'First', 'items': [], 'meta': {'start': 1}},
                {'content': 'Second', 'items': [], 'meta': {'start': 1}},
            ]
        )
        result = data.render_to_text()
        assert '1. First' in result

    def test_render_to_text_checklist(self):
        data = ListEditorBlockData(
            style=ListEditorBlockDataStyle.CHECKLIST,
            items=[
                {'content': 'Done', 'items': [], 'meta': {'checked': True}},
                {'content': 'Not done', 'items': [], 'meta': {'checked': False}},
            ]
        )
        result = data.render_to_text()
        assert '[X] Done' in result
        assert '[ ] Not done' in result

    def test_render_to_text_nested(self):
        data = ListEditorBlockData(
            style=ListEditorBlockDataStyle.UNORDERED,
            items=[
                {
                    'content': 'Parent',
                    'items': [
                        {'content': 'Child', 'items': []}
                    ]
                },
            ]
        )
        result = data.render_to_text()
        assert 'Parent' in result
        assert 'Child' in result



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from playwright.sync_api import Locator, Page


class Card:
    """Playwright component for django_spire/card/card.html"""

    def __init__(self, page: Page, card_selector: str = '.card') -> None:
        self.card_selector = card_selector
        self.page = page

    @property
    def card(self) -> Locator:
        return self.page.locator(self.card_selector).first

    @property
    def content(self) -> Locator:
        return self.card.locator('.card-body, .p-3').first

    def is_visible(self) -> bool:
        return self.card.is_visible()


class TitleCard(Card):
    """Playwright component for django_spire/card/title_card.html"""

    @property
    def button(self) -> Locator:
        return self.card.locator('.col-auto.d-flex').first

    @property
    def dropdown_content(self) -> Locator:
        return self.card.locator('[x-show="card_title_dropdown"]')

    @property
    def title(self) -> Locator:
        return self.card.locator('.card-title').first

    def click_button(self) -> None:
        self.button.locator('button, a').first.click()

    def get_title_text(self) -> str:
        return self.title.inner_text()

    def has_button(self) -> bool:
        return self.button.locator('button, a').count() > 0

    def is_dropdown_open(self) -> bool:
        return self.dropdown_content.is_visible()

    def toggle_dropdown(self) -> None:
        self.card.locator('[\\@click*="toggle_card_title_dropdown"]').click()


class FormCard(TitleCard):
    """Playwright component for django_spire/card/form_card.html"""

    @property
    def description(self) -> Locator:
        return self.card.locator('.mb-3').first

    @property
    def form(self) -> Locator:
        return self.card.locator('form')

    def fill_field(self, name: str, value: str) -> None:
        self.form.locator(f'[name="{name}"]').fill(value)

    def get_field_value(self, name: str) -> str:
        return self.form.locator(f'[name="{name}"]').input_value()

    def submit(self) -> None:
        self.form.locator('button[type="submit"], input[type="submit"]').click()


class InfiniteScrollCard(TitleCard):
    """Playwright component for django_spire/card/infinite_scroll_card.html"""

    @property
    def loaded_count_text(self) -> Locator:
        return self.card.locator('[x-text="loaded_count"]')

    @property
    def scroll_container(self) -> Locator:
        return self.card.locator('[x-ref="scroll_container"]')

    @property
    def total_count_text(self) -> Locator:
        return self.card.locator('[x-text="total_count"]')

    def get_loaded_count(self) -> int:
        return int(self.loaded_count_text.inner_text())

    def get_total_count(self) -> int:
        return int(self.total_count_text.inner_text())

    def scroll_to_bottom(self) -> None:
        self.scroll_container.evaluate('el => el.scrollTop = el.scrollHeight')



from __future__ import annotations

from django import template


register = template.Library()


@register.filter
def dashes_to_underscore(value: str) -> str:
    return value.replace('-', '_')


@register.filter
def spaces_to_underscore(value: str) -> str:
    return value.replace(' ', '_')


@register.filter
def dashes_and_spaces_to_underscore(value: str) -> str:
    return spaces_to_underscore(dashes_to_underscore(value))


@register.filter
def underscores_to_spaces(value: str) -> str:
    return value.replace('_', ' ')



from __future__ import annotations

# import json

from django.contrib.sites.models import Site
# from django.conf import settings
# from django.test import RequestFactory

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.collection.models import Collection
from django_spire.knowledge.collection.tests.factories import create_test_collection


class CollectionServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.collection = create_test_collection()

    def test_save_model_obj_creates_new(self):
        collection, created = Collection.services.save_model_obj(
            name='New Collection',
            description='A new collection'
        )
        assert created is True
        assert collection.name == 'New Collection'

    def test_save_model_obj_updates_existing(self):
        self.collection.services.obj = self.collection
        updated_collection, created = self.collection.services.save_model_obj(
            name='Updated Name'
        )
        assert created is False
        assert updated_collection.name == 'Updated Name'


class CollectionOrderingServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.parent = create_test_collection(name='Parent')
        self.child1 = create_test_collection(parent=self.parent, name='Child 1')
        self.child2 = create_test_collection(parent=self.parent, name='Child 2')

    def test_reorder_within_same_parent(self):
        self.child2.services.ordering.reorder(order=0, new_parent_pk=self.parent.pk)
        self.child1.refresh_from_db()
        self.child2.refresh_from_db()
        assert self.child2.order <= self.child1.order

    def test_reorder_to_new_parent(self):
        new_parent = create_test_collection(name='New Parent')
        self.child1.services.ordering.reorder(order=0, new_parent_pk=new_parent.pk)
        self.child1.refresh_from_db()
        assert self.child1.parent_id == new_parent.pk

    def test_reorder_to_root(self):
        self.child1.services.ordering.reorder(order=0, new_parent_pk=None)
        self.child1.refresh_from_db()
        assert self.child1.parent_id is None


class CollectionProcessorServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.parent = create_test_collection(name='Parent')
        self.child = create_test_collection(parent=self.parent, name='Child')

    def test_set_deleted(self):
        self.parent.services.processor.set_deleted()
        self.parent.refresh_from_db()
        assert self.parent.is_deleted is True

    def test_set_deleted_cascades_to_children(self):
        self.parent.services.processor.set_deleted()
        self.child.refresh_from_db()
        assert self.child.is_deleted is True


class CollectionToolServiceTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.parent = create_test_collection(name='Parent')
        self.child = create_test_collection(parent=self.parent, name='Child')
        self.grandchild = create_test_collection(parent=self.child, name='Grandchild')

    def test_get_children_ids(self):
        children_ids = Collection.services.tool.get_children_ids(parent_id=self.parent.pk)
        assert self.child.pk in children_ids
        assert self.grandchild.pk in children_ids
        assert self.parent.pk not in children_ids

    def test_get_root_collection_pk(self):
        self.grandchild.services.tool.obj = self.grandchild
        root_pk = self.grandchild.services.tool.get_root_collection_pk()
        assert root_pk == self.parent.pk

    def test_get_root_collection_pk_when_already_root(self):
        self.parent.services.tool.obj = self.parent
        root_pk = self.parent.services.tool.get_root_collection_pk()
        assert root_pk == self.parent.pk


class TestCollectionTransformationService(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.site = Site.objects.create(domain='', name='')

    def _test_collection_urls_json(self, collection_json: dict):
        response = self.client.get(collection_json['delete_url'].replace(' ', ''))
        assert response.status_code == 200

        response = self.client.get(collection_json['create_entry_url'].replace(' ', ''))
        assert response.status_code == 200

        response = self.client.get(collection_json['import_entry_url'].replace(' ', ''))
        assert response.status_code == 200

    # # TODO(Tyrell): Talk to Chase about this
    # def test_to_hierarchy_json(self):
    #     settings.SITE_ID = self.site.id

    #     self.test_collection_1 = Collection.objects.create(name='Grand Parent A', id=1, parent_id=None)
    #     self.test_collection_2 = Collection.objects.create(name='Parent A1', id=2, parent_id=1)
    #     self.test_collection_3 = Collection.objects.create(name='Child A1a', id=3, parent_id=2)

    #     request = RequestFactory().get('/')
    #     request.user = self.super_user

    #     family_tree = Collection.services.transformation.to_hierarchy_json(
    #         request=request
    #     )

    #     for collection_json in json.loads(family_tree):
    #         assert collection_json['id'] is not None
    #         assert collection_json['name'] == 'Grand Parent A'

    #         assert len(collection_json['entries']) == 0

    #         self._test_collection_urls_json(collection_json=collection_json)

    #         assert len(collection_json['children']) == 1

    #         for child_json in collection_json['children']:
    #             assert child_json['id'] is not None
    #             assert child_json['name'] == 'Parent A1'

    #             assert len(child_json['entries']) == 0

    #             self._test_collection_urls_json(collection_json=child_json)

    #             assert len(child_json['children']) == 1

    #             for grand_child_json in child_json['children']:
    #                 assert grand_child_json['id'] is not None
    #                 assert grand_child_json['name'] == 'Child A1a'

    #                 assert len(grand_child_json['entries']) == 0

    #                 self._test_collection_urls_json(collection_json=grand_child_json)

    #                 assert len(grand_child_json['children']) == 0



from __future__ import annotations

from django.urls import path

from django_spire.notification.app.views import template_views


app_name = 'django_spire_notification'

urlpatterns = [
    path('notficiation/dropdown/template/',
        template_views.notification_dropdown_template_view,
        name='notification_dropdown')
]
